﻿  ; Load the Autocorrect list from a .XML file
  ;
  ; V1.0 - 6/JAN/2016
  ;        7/JAN/2016
  ;        9/JAN/2016
  ; V1.1 - 26/DEC/2016
  ;        o Started implementing the multiselection. 
  ; V1.2 - 8/JUN/2017
  ;        o undo/redo support  
  ; V1.3 - 12/JUN/2017
  ;        o Autocorrect: Allow to try to open a file with a non-valid header
  ; V1.4 - 31/OCT/2017
  ;        o Support for "Recent Files" menu entry
  ; V1.5 - 1/DEC/2017
  ;        o Message Requesters now have a sign.
  ; V1.6 - 10/FEB/2020
  ;        o Now reads BOM files (3-FEB added) properly without the warning visible on Autocorrect.
  ; V1.7 - 15/FEB/2020
  ;        o All file operations support BOM.
  ; V1.8 - 06/JAN/2021
  ;        o Options that use file open show on the status bar the filesize after "OK."
  ; V1.9 - 02/JUL/2021
  ;        o Now the sizes of load/save main files for each tab shows the files label. Ex: "(aff:1 MB)+(dic: 10MB)"  
  ; V2.0 - 22/NOV/2021
  ;        23/NOV/2021
  ;        o Started showing the time taken in timely operations.
  ;
  Procedure load_autocorrect_list(force_file$)
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Loading Autocorrect List... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
    
    ; Confirm if to load a new Autocorrect List if the file in memory has been changed
    If autocorrect_changed=#True
      If MessageRequester("Warning!","The Autocorrect List has been changed."+#LF$+#LF$+"Continue?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Info)=#PB_MessageRequester_No : ProcedureReturn #False : EndIf
    EndIf    
    
    
    If force_file$=""
      file$=OpenFileRequester("Open Autocorrect List", "", "Autocorrect List (*.xml)|*.xml", 0)
      If file$=""
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")  
        ProcedureReturn #False
      EndIf
    Else
      file$=force_file$
    EndIf
    
    
    ; Check if .XML file exists
    If IsFileExist(file$,"",".XML file not found.")=#False
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "File not found.")
      ProcedureReturn #False
    EndIf    
    
    autocorrect_number_header_lines=1
    
    ; Check If .XML matches the needed header (which presumes UTF-8)
    ReadFile(1,file$)
    ReadStringFormat(1)
    text$=ReadString(1,#PB_UTF8)
    CloseFile(1)
    If LCase(text$)<>LCase("<?xml version="+Chr(34)+"1.0"+Chr(34)+" encoding="+Chr(34)+"UTF-8"+Chr(34)+" ?> <block-list:block-list xmlns:block-list="+Chr(34)+"http://openoffice.org/2001/block-list"+Chr(34)+">")
      If MessageRequester("Warning!","The first line of an Autocorrect List must be:"+#LF$+"<?xml version="+Chr(34)+"1.0"+Chr(34)+" encoding="+Chr(34)+"UTF-8"+Chr(34)+" ?> <block-list:block-list xmlns:block-list="+Chr(34)+"http://openoffice.org/2001/block-list"+Chr(34)+">"+#LF$+#LF$+"Attempt to import the file?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
        ProcedureReturn #False
      EndIf
      flag=#False
      ReadFile(1,file$)
      ReadStringFormat(1)
      Repeat
        text$=ReadString(1,#PB_UTF8)
        If CountString(LCase(text$),"<block-list:block")>0
          flag=#True
          Break
        EndIf
        autocorrect_number_header_lines+1
      Until Eof(1)      
      CloseFile(1)
      If flag=#False
        MessageRequester("Warning!","Failed to import file.",#PB_MessageRequester_Warning)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
        ProcedureReturn #False        
      EndIf
      autocorrect_number_header_lines-1
    EndIf
      
    
    autocorrect_disable_undo_redo()    
      
    selected_autocorrect_word=-1 ; Reset the current Autocorrect word selected
    autocorrect_changed=#False ; Autocorrect List set to unchanged               
    ; Read AUTOCORRECT LIST file
    time_taken = ElapsedMilliseconds()   
    If number_of_autocorrect_words>0 : ClearGadgetItems(#PANEL_BOX_AUTOCORRECT) : EndIf ; To remove the current words from the gadgetlist    
    AddWindowTimer(#WINDOW_MAIN,800,800)      
    ReadFile(1,file$)
      ReadStringFormat(1)
      number_of_autocorrect_words=0
      Dim autocorrect_words$(0)
      Dim autocorrect_words_selected(0)
      If autocorrect_number_header_lines>0
        For f=1 To autocorrect_number_header_lines
          text$=ReadString(1,#PB_UTF8) ;Just to read the header of the .XML
        Next f
      EndIf
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,Lof(1))
      Repeat
        word$=ReadString(1,#PB_UTF8)
        
        If CountString(LCase(word$),"<block-list:block")>1
          MessageRequester("Warning!","Only allowed one pair of incorrect/correct per line."+#LF$+"Use a program like Notepad++ to create individual lines."+#LF$+"For example: replace "+Chr(34)+"/>"+Chr(34)+" with "+Chr(34)+"/>\n"+Chr(34)+#LF$+"using Extended search mode.",#PB_MessageRequester_Warning)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          CloseFile(1)
          RemoveWindowTimer(#WINDOW_MAIN,800)           
          ProcedureReturn #False
        EndIf
          
        
        If word$<>LCase("</block-list:block-list>")
          ReDim autocorrect_words$(number_of_autocorrect_words)    
          ReDim autocorrect_words_selected(number_of_autocorrect_words)
          autocorrect_words$(number_of_autocorrect_words)=StringField(word$,2,Chr(34))+#LF$+StringField(word$,4,Chr(34))
          number_of_autocorrect_words+1
        EndIf
        
        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event=WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
          update_number_autocorrect_words()
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Loading Autocorrect List... <ESC> to abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,Loc(1),#PB_StatusBar_Raised)
        EndIf 
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
         If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            CloseFile(1)
            RemoveWindowTimer(#WINDOW_MAIN,800)           
            ProcedureReturn #False
         EndIf
        EndIf

      Until Eof(1)
    filesize_lof=Lof(1)
    CloseFile(1)  
    update_number_autocorrect_words()
    RemoveWindowTimer(#WINDOW_MAIN,800)
    autocorrect_words_filename$=file$
    update_filename_autocorrect_list()
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 21/MAR/2014
      
    ; New code to handle the vertical scroll bar
    SetGadgetAttribute(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_autocorrect_words-1)
    SetGadgetState(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,0)
    global_update_virtual_vertical_scroll_bar(-1)
        
    ; 31/OCT/2017 - Support for "Recent Files" menu entry
    create_load_update_recent_files(file$)
    update_recent_files_menu()    
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK. "+ShowFileSizeFormatted("xml:",filesize_lof,#True)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    
  EndProcedure
  
  
  
  
  ; Save/Save As the autocorrect words to a .XML file
  ; V1.0 - 7/JAN/2016
  ;        9/JAN/2016
  ; V1.1 - 11/SEP/2016
  ;        o Now it saves Dictionary + Thesaurus + Autocorrect with #LF$ instead of #CRLF for Linux compatibility
  ; V1.2 - 31/OCT/2017
  ;        o Support for "Recent Files" menu entry
  ; V1.3 - 1/DEC/2017
  ;        o Message Requesters now have a sign.  
  ; V1.4 - 03/FEB/2020
  ;        o Autocorrect now has a BOM
  ; V1.5 - 02/DEC/2020
  ;        o Options that use file save/extract shown on the status bar show the filesize after "Okay."
  ; V1.6 - 02/JUL/2021
  ;        o Now the sizes of load/save main files for each tab shows the files label. Ex: "(aff:1 MB)+(dic: 10MB)"  
  ; V1.7 - 25/AUG/2021
  ;        o Added spaces on left regarding autocorrect (prefs + core)
  ; V1.8 - 22/NOV/2021
  ;        23/NOV/2021
  ;        o Started showing the time taken in timely operations.
  ;
  Procedure save_saveas_autocorrect_list(save, saveas)
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Saving Autocorrect List... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) 
    
    If number_of_autocorrect_words=0
      MessageRequester("Warning!","No autocorrect words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf
    
    
    If save=1 : file$=autocorrect_words_filename$ : EndIf
    
    If save=1 And autocorrect_words_filename$="" : saveas=1 : EndIf
    
    If saveas=1
      file$=SaveFileRequester("Save Autocorrect List", "", "Autocorrect List (*.xml)|*.xml", 0)
      If file$=""
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        ProcedureReturn #False
      EndIf
      If LCase(Right(file$,4))<>".xml" : file$+".xml": EndIf
    EndIf
    

    If FileSize(file$)>0 And saveas=1
      If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        ProcedureReturn #False
      EndIf      
    EndIf    
    
    DeleteFile(file$) ; Delete .XML file
                 

    ; Saving the .XML file
    AddWindowTimer(#WINDOW_MAIN,800,800)
    OpenFile(1,file$)
      WriteStringFormat(1,#PB_UTF8)
      WriteString(1,"<?xml version="+Chr(34)+"1.0"+Chr(34)+" encoding="+Chr(34)+"UTF-8"+Chr(34)+" ?> <block-list:block-list xmlns:block-list="+Chr(34)+"http://openoffice.org/2001/block-list"+Chr(34)+">"+#LF$,#PB_UTF8) ; Write the header
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_autocorrect_words)     
      time=ElapsedMilliseconds()
      time_taken=ElapsedMilliseconds()
      For f=0 To number_of_autocorrect_words-1
        text$=autocorrect_words$(f)
        WriteString(1,Space(autocorrect_spaces_on_left)+"<block-list:block block-list:abbreviated-name="+Chr(34)+StringField(text$,1,#LF$)+Chr(34)+" block-list:name="+Chr(34)+StringField(text$,2,#LF$)+Chr(34)+"/>"+#LF$,#PB_UTF8)
                      
        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event = WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Saving Autocorrect List... <ESC> to abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        EndIf 
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
          If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            CloseFile(1)
            RemoveWindowTimer(#WINDOW_MAIN,800)
            ProcedureReturn #False
         EndIf
        EndIf
        

      Next f
      WriteString(1,"</block-list:block-list>"+#LF$,#PB_UTF8) ; Write the footer
      RemoveWindowTimer(#WINDOW_MAIN,800)      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      filesize_lof=Lof(1)
      CloseFile(1)
      
      ; Autocorrect List set to unchanged
      autocorrect_changed=#False
      update_number_autocorrect_words()
      
      ; 31/OCT/2017 - Support for "Recent Files" menu entry
      create_load_update_recent_files(file$)
      update_recent_files_menu()      
      
      autocorrect_words_filename$=file$
      update_filename_autocorrect_list()
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK. "+ShowFileSizeFormatted("xml:",filesize_lof,#True)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    
  EndProcedure
  
  
  
  
  ; Updates the number of words in the Autocorrect List
  ; Useful in open, add, delete
  ; V1.0 - 6/JAN/2016
  ; V1.1 - 25/OCT/2017
  ;        o Formats the total number of items to be better seen.  
  ; V1.2 - 20/JUL/2018
  ;        o Now the changes allow a colour asterisk in red 
  ; V1.3 - 23/JUL/2018
  ;        o Fix: Asterisks in red no longer flash  
  ; V1.4 - 27/JUL/2018
  ;        28/JUL/2018
  ;        o Fix: Reduced asterisks vanishing if scrolling holding down the arrow keys
  ; V1.5 - 19/SEP/2018
  ;        o Floppy icon on tabs (blue/red)
  ;
  Procedure update_number_autocorrect_words()
    
    ; Black/Red floppy
    If autocorrect_changed=#False And GetGadgetText(#TEXT_PANEL_BOX_AUTOCORRECT_NUMBER_WORDS_ASTERISK)<>""
      SetGadgetItemImage(#PANEL_MAIN,3,ImageID(71))
    EndIf
    If autocorrect_changed=#True And GetGadgetText(#TEXT_PANEL_BOX_AUTOCORRECT_NUMBER_WORDS_ASTERISK)<>"*"
      SetGadgetItemImage(#PANEL_MAIN,3,ImageID(72))
    EndIf  
    
    t$=""
    If autocorrect_changed=#True : t$="*" : EndIf        
    If selected_autocorrect_word=-1
      t1$="# Autocorrects:"+FormatNumber(number_of_autocorrect_words,0,".",number_separator_chr$)
    Else
      t1$="Autocorrect: ["+FormatNumber(selected_autocorrect_word+1,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_autocorrect_words,0,".",number_separator_chr$)+"]"
    EndIf   
       
    If t1$<>GetGadgetText(#TEXT_PANEL_BOX_AUTOCORRECT_NUMBER_WORDS)
      gadget=#TEXT_PANEL_BOX_AUTOCORRECT_NUMBER_WORDS
      gadget2=#TEXT_PANEL_BOX_AUTOCORRECT_NUMBER_WORDS_ASTERISK
      w=GadgetWidth(gadget)
      SetGadgetText(gadget,t1$)
      GadgetFit(gadget)
      If GadgetX(gadget2)<GadgetX(gadget)+GadgetWidth(gadget) Or GadgetX(gadget2)>=GadgetX(gadget)+GadgetWidth(gadget)+4
        ResizeGadget(gadget2,GadgetX(gadget)+GadgetWidth(gadget),GadgetY(gadget),16,16)                        
        GadgetRefresh(gadget2)  
      EndIf
    EndIf     
    
    If GetGadgetText(#TEXT_PANEL_BOX_AUTOCORRECT_NUMBER_WORDS_ASTERISK)<>t$
      SetGadgetText(#TEXT_PANEL_BOX_AUTOCORRECT_NUMBER_WORDS_ASTERISK,t$)
    EndIf
      
  EndProcedure  
  
  
  
  
  ; Updates the filename of the words in the Autocorrect List
  ; Used in open and save as
  ; V1.0 - 6/JAN/2016
  ; V1.2 - 11/FEB/2016
  ;        o Simplified the code
  ; V1.3 - 25/FEB/2020
  ;        o Fix: Long filenames now fit in the GUI
  ; V1.4 - 26/FEB/2020
  ;        o Cleaned the code that updates filenames in the GUI  
  ; V1.5 - 19/DEC/2021
  ;        o Replaced "Filename:" with "File:"
  ; V1.6 - 01/NOV/2023
  ;        o Replaced "File:" with "💾:"
  ; V1.7 - 26/NOV/2023
  ;        o Replaced the disk emoji plain text with an image
  ; V1.8 - 13/JAN/2024
  ;        14/JAN/2024
  ;        o It now shows a folder emoji with the path of the filename
  ;
  Procedure update_filename_autocorrect_list()
    
    TextReplaceIfDifferentGadget(#TEXT_PANEL_BOX_AUTOCORRECT_FILENAME,""+GetOnlyFilename(autocorrect_words_filename$,"n/a"),#True)
    TextReplaceIfDifferentGadget(#TEXT_PANEL_BOX_AUTOCORRECT_FOLDER,""+GetOnlyPath(autocorrect_words_filename$,"n/a"),#True)

  EndProcedure  
  
  
  
  
  ; ADD/EDIT AUTOCORRECT WORDS
  ; If add=1 and edit=0 then it ADDS
  ; If add=0 and edit=1 then it EDITS
  ;
  ; V1.0 - 7/JAN/2016
  ; V1.1 - 26/FEB/2016
  ;        o In "Autocorrect" we now have red and green text.
  ; V1.2 - 23/OCT/2016
  ;        o Replaced the red and green text in the Autocorrect editor, with the wrong and right symbols.
  ; V1.3 - 26/DEC/2016
  ;        o Started implementing the multiselection.  
  ; V1.4 - 11/MAR/2017
  ;        o Changed the buttons height to 25.  
  ; V1.5 - 8/JUN/2017
  ;        o undo/redo support  
  ; V1.6 - 23/JUL/2017
  ;        o F1 (help) now works in the edit/new items windows
  ; V1.7 - 01/DEC/2017
  ;        o Message Requesters now have a sign. 
  ; V1.8 - 11/APR/2019
  ;        o Autocorrect string gadgets are now larger
  ; V1.9 - 29/MAY/2019
  ;        o GTK3 Ready
  ; V2.0 - 20/JUN/2019
  ;        o Preferences allow to choose the font for the language.  
  ; V2.1 - 11/JUL/2019
  ;        o New wrong/right images in the Autocorrect UI
  ; V2.2 - 11/FEB/2022
  ;        o Edit autocorrect, the text boxes now have red ink for incorrect and green ink for correct
  ; V2.3 - 01/NOV/2022
  ;        o Fix: Edit autocorrect now changes the ink colour to black if field flagged to red
  ; V2.4 - 27/DEC/2022
  ;        o If incorrect=correct it popsup a warning: "Incorrect matches correct."
  ; V2.5 - 10/FEB/2023
  ;        o Autocorrect edit: added toggle text up/down (incorrect/correct) button
  ;
  Procedure add_edit_autocorrect(add,edit)
    
    If number_of_autocorrect_words=0 And edit=#True
      MessageRequester("Warning!","No autocorrect words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf    
    
    t=selected_autocorrect_word
    If t=-1 And edit=#True
      MessageRequester("Warning!","No autocorrect word selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
         
    t$="Add an autocorrect word" ;add=#True
    If edit=#True : t$="Edit an autocorrect word" : EndIf
    w=(800-640)+640-10-100+20+20
    h=2+3-10-20-40-50-200+200-50-20-20+100+100+200-30-30-30+10+20+5
    If OpenWindow(#WINDOW_ADD_EDIT,0,0,w,h,t$,#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    DisableWindow(#WINDOW_MAIN,#True)
        
    ; Incorrect field    
    t=selected_autocorrect_word
    t$=""
    If t<>-1 And edit=#True
      t$=StringField(autocorrect_words$(t),1,#LF$)
    EndIf
    x=-25+10+0+6+100-15-15-20
    y=6+0+6
    StringGadget(#STRING_WINDOW_ADD_EDIT_INCORRECT,x,y,20+320-10-20-40+20+30+30+60+70+70+70,default_gadget_height_string,t$)
    SetGadgetColor(#STRING_WINDOW_ADD_EDIT_INCORRECT,#PB_Gadget_FrontColor,#Red)
    SetActiveGadget(#STRING_WINDOW_ADD_EDIT_INCORRECT)
    old_incorrect$=t$ ; 5/NOV/2014
    SetGadgetFont(#STRING_WINDOW_ADD_EDIT_INCORRECT,FontID(#font_for_language_id))
    ImageGadget(#IMAGE_WINDOW_ADD_EDIT_INCORRECT,x-24-5-2,y+3,24,24,ImageID(83))
    
    ; Correct field
    t$=""
    If t<>-1 And edit=#True
      t1=FindString(autocorrect_words$(t),#LF$)
      If t1>0 : t$=StringField(autocorrect_words$(t),2,#LF$) : EndIf
    EndIf    
    y+default_gadget_height_string+5
    StringGadget(#STRING_WINDOW_ADD_EDIT_CORRECT,x,y,20+320-10-20-40+20+30+30+60+70+70+70,default_gadget_height_string,t$) 
    SetGadgetColor(#STRING_WINDOW_ADD_EDIT_CORRECT,#PB_Gadget_FrontColor,#GreenDark)
    old_correct$=t$ ; 5/NOV/2014
    SetGadgetFont(#STRING_WINDOW_ADD_EDIT_CORRECT,FontID(#font_for_language_id))
    ImageGadget(#IMAGE_WINDOW_ADD_EDIT_CORRECT,x-24-5,y+3,24,24,ImageID(84))
    
    
    ; Add the move up to down sentence
    x=GadgetX(#STRING_WINDOW_ADD_EDIT_INCORRECT)+GadgetWidth(#STRING_WINDOW_ADD_EDIT_INCORRECT)+10
    y=GadgetY(#STRING_WINDOW_ADD_EDIT_INCORRECT)+default_gadget_height_string-(default_gadget_width_button_image_24x24/2)
    ButtonImageGadget(#BUTTON_AUTOCORRECT_TOGGLE_UP_DOWN,x,y,default_gadget_width_button_image_24x24,default_gadget_width_button_image_24x24,ImageID(113))
    GadgetToolTip(#BUTTON_AUTOCORRECT_TOGGLE_UP_DOWN,"Toggle text Incorrect↔Correct")
    
    
    ; Add the OK and CANCEL buttons
    ButtonGadget(#BUTTON_ADD_EDIT_OKAY,0+6+100-15,h-10-default_gadget_height_button,90,default_gadget_height_button,"OK")
    ButtonGadget(#BUTTON_ADD_EDIT_CANCEL,0+6+100-15+100,h-10-default_gadget_height_button,90,default_gadget_height_button,"Cancel")
        
    ; Wait for the user to press the OK or CANCEL button
    flag=#False ;to be used if text box with words is changed. #False=unchanged, #True=changed
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_ADD_EDIT,#PB_Shortcut_Escape,1000)    
    AddKeyboardShortcut(#WINDOW_ADD_EDIT,#PB_Shortcut_Return,1003)    
    AddKeyboardShortcut(#WINDOW_ADD_EDIT,#PB_Shortcut_F1,1004)
    
    okay=#False
    cancel=#False
    Repeat
      
      
      ; Change background of word string gadget if a space is found at the start or end, or two spaces in a row (invalid word)  
      t$=GetGadgetText(#STRING_WINDOW_ADD_EDIT_INCORRECT)
      If t$<>"" And (Left(t$,1)=" " Or Right(t$,1)=" " Or FindString(t$,"  ")<>0)
        If GetGadgetColor(#STRING_WINDOW_ADD_EDIT_INCORRECT,#PB_Gadget_BackColor)<>#Red
          SetGadgetColor(#STRING_WINDOW_ADD_EDIT_INCORRECT,#PB_Gadget_BackColor,#Red)
          SetGadgetColor(#STRING_WINDOW_ADD_EDIT_INCORRECT,#PB_Gadget_FrontColor,#Black)
        EndIf
      Else
        If GetGadgetColor(#STRING_WINDOW_ADD_EDIT_INCORRECT,#PB_Gadget_BackColor)<>-1
          SetGadgetColor(#STRING_WINDOW_ADD_EDIT_INCORRECT,#PB_Gadget_BackColor,-1)
          SetGadgetColor(#STRING_WINDOW_ADD_EDIT_INCORRECT,#PB_Gadget_FrontColor,#Red)
        EndIf
      EndIf    
      
      ; Change background of word string gadget if a space is found at the start or end, or two spaces in a row (invalid word)  
      t$=GetGadgetText(#STRING_WINDOW_ADD_EDIT_CORRECT)
      If t$<>"" And (Left(t$,1)=" " Or Right(t$,1)=" " Or FindString(t$,"  ")<>0)
        If GetGadgetColor(#STRING_WINDOW_ADD_EDIT_CORRECT,#PB_Gadget_BackColor)<>#Red
          SetGadgetColor(#STRING_WINDOW_ADD_EDIT_CORRECT,#PB_Gadget_BackColor,#Red)
          SetGadgetColor(#STRING_WINDOW_ADD_EDIT_CORRECT,#PB_Gadget_FrontColor,#Black)
        EndIf
      Else
        If GetGadgetColor(#STRING_WINDOW_ADD_EDIT_CORRECT,#PB_Gadget_BackColor)<>-1
          SetGadgetColor(#STRING_WINDOW_ADD_EDIT_CORRECT,#PB_Gadget_BackColor,-1)
          SetGadgetColor(#STRING_WINDOW_ADD_EDIT_CORRECT,#PB_Gadget_FrontColor,#GreenDark)
        EndIf
      EndIf             
      
      
      event=WaitWindowEvent()
      
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf     
      
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        cancel=#True
      EndIf        
      
      
      ; Check if <ENTER> was pressed
      If event=#PB_Event_Menu And EventMenu()=1003
        Debug "OK was pressed"
        PostEvent(#PB_Event_Gadget, #WINDOW_ADD_EDIT, #BUTTON_ADD_EDIT_OKAY)
      EndIf      
      
      
      ; Shortcut for F1 (help)
      If event=#PB_Event_Menu And EventMenu()=1004
        html_help("")
      EndIf         
    
    
      Select Event 
        Case #PB_Event_Gadget
          Select EventGadget()
              
           ; Press toggle up/down
           Case #BUTTON_AUTOCORRECT_TOGGLE_UP_DOWN
             t1$=GetGadgetText(#STRING_WINDOW_ADD_EDIT_INCORRECT)
             t2$=GetGadgetText(#STRING_WINDOW_ADD_EDIT_CORRECT)
             Swap t1$,t2$
             SetGadgetText(#STRING_WINDOW_ADD_EDIT_INCORRECT,t1$)
             SetGadgetText(#STRING_WINDOW_ADD_EDIT_CORRECT,t2$)
             
             
           ; Press OK
           Case #BUTTON_ADD_EDIT_OKAY
             okay=#True
             t$=GetGadgetText(#STRING_WINDOW_ADD_EDIT_INCORRECT)
             If (t$="" Or t$=" ") And okay=#True
               MessageRequester("Warning!","No incorrect word text.",#PB_MessageRequester_Warning)
               SetActiveGadget(#STRING_WINDOW_ADD_EDIT_INCORRECT)
               okay=#False
             EndIf                   
             If (CountString(t$,"  ") Or Left(t$,1)=" " Or Right(t$,1)=" ") And okay=#True
               MessageRequester("Warning!","Invalid spaces in incorrect word.",#PB_MessageRequester_Warning)
               SetActiveGadget(#STRING_WINDOW_ADD_EDIT_INCORRECT)
               okay=#False
             EndIf
             
             t$=GetGadgetText(#STRING_WINDOW_ADD_EDIT_CORRECT)
             If (t$="" Or t$=" ") And okay=#True
               MessageRequester("Warning!","No correct word text.",#PB_MessageRequester_Warning)
               SetActiveGadget(#STRING_WINDOW_ADD_EDIT_CORRECT)
               okay=#False
             EndIf                   
             If (CountString(t$,"  ") Or Left(t$,1)=" " Or Right(t$,1)=" ") And okay=#True
               MessageRequester("Warning!","Invalid spaces in correct word.",#PB_MessageRequester_Warning)
               SetActiveGadget(#STRING_WINDOW_ADD_EDIT_CORRECT)
               okay=#False
             EndIf
             
             If GetGadgetText(#STRING_WINDOW_ADD_EDIT_INCORRECT)=GetGadgetText(#STRING_WINDOW_ADD_EDIT_CORRECT)
               MessageRequester("Warning!","Incorrect matches correct.",#PB_MessageRequester_Warning)
               SetActiveGadget(#STRING_WINDOW_ADD_EDIT_CORRECT)
               okay=#False
            EndIf
             
           ; Press CANCEL  
           Case #BUTTON_ADD_EDIT_CANCEL
             cancel=#True             
             
          EndSelect
       EndSelect                 
                 
      
    Until okay=#True Or cancel=#True
    
    ; OK button pressed. It will now add or update an autocorrect word    
    If okay=#True      
      
      If old_incorrect$<>GetGadgetText(#STRING_WINDOW_ADD_EDIT_INCORRECT) Or old_correct$<>GetGadgetText(#STRING_WINDOW_ADD_EDIT_CORRECT)
        autocorrect_store_undo(#False) ; 8/JUN/2017
      EndIf       
            
      t$=GetGadgetText(#STRING_WINDOW_ADD_EDIT_INCORRECT)+#LF$+GetGadgetText(#STRING_WINDOW_ADD_EDIT_CORRECT)

      If add=#True
        ReDim autocorrect_words$(number_of_autocorrect_words)        
        ReDim autocorrect_words_selected(number_of_autocorrect_words) 
        autocorrect_words$(number_of_autocorrect_words)=t$
        number_of_autocorrect_words+1
        SetGadgetAttribute(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_autocorrect_words-1)
        update_number_autocorrect_words()
        global_update_virtual_vertical_scroll_bar(-1)
      EndIf
      If edit=#True
        t=selected_autocorrect_word         
        autocorrect_words$(t)=t$
        global_update_virtual_vertical_scroll_bar(-1)
      EndIf
    EndIf
    
    ; OK pressed and change the autocorrect_changed variable (ONLY IF WORD/TAGS WERE CHANGED - 5/NOV/2014)
    If okay=#True
      If old_incorrect$<>GetGadgetText(#STRING_WINDOW_ADD_EDIT_INCORRECT) Or old_correct$<>GetGadgetText(#STRING_WINDOW_ADD_EDIT_CORRECT)
        autocorrect_changed=#True
      EndIf 
      update_number_autocorrect_words()
    EndIf    
    
    update_vertical_scroll_bar_autocorrect_list_current_item()
    
    ; Close the Add/Edit window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    CloseWindow(#WINDOW_ADD_EDIT) 
    
  EndProcedure 

    
  
  
  ; Delete words in the Autocorrect List
  ;
  ; V1.0 - 7/JAN/2016
  ; V1.1 - 26/DEC/2016
  ;        o Started implementing the multiselection. 
  ; V1.2 - 8/JUN/2017
  ;        o undo/redo support 
  ; V1.3 - 6/NOV/2017
  ;        o Now recognises being called from advanced find
  ; V1.4 - 1/DEC/2017
  ;        o Message Requesters now have a sign.  
  ;
  Procedure delete_autocorrect(is_called_from_advance_find)
    If number_of_autocorrect_words=0 And is_called_from_advance_find=#False
      MessageRequester("Warning!","No autocorrect words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    t=selected_autocorrect_word
    If number_of_autocorrect_words>0 And t=-1 And is_called_from_advance_find=#False
      MessageRequester("Warning!","No autocorrect word selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    If number_of_autocorrect_words>0 And t<>-1      
      text$=autocorrect_words$(t)
      text$=ReplaceString(text$,#LF$," ")
      If MessageRequester("Warning!","Really delete autocorrect word?"+#LF$+Chr(34)+text$+Chr(34),#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes      
        flag=#False 
        
        autocorrect_store_undo(#False) ; 8/JUN/2017
        
        ; Only one word in the autocorrect list
        If number_of_autocorrect_words=1 And flag=#False
          ReDim autocorrect_words$(0)
          ReDim autocorrect_words_selected(0)
          autocorrect_words_selected(0)=#False
          flag=#True
        EndIf
        
        ; The word to delete is the last
        If number_of_autocorrect_words=t+1 And flag=#False
          ReDim autocorrect_words$(ArraySize(autocorrect_words$())-1)
          ReDim autocorrect_words_selected(ArraySize(autocorrect_words_selected())-1)
          flag=#True
        EndIf
        
        ; The word to delete is inside the list, not the first nor the last item
        If flag=#False
          For f=t+1 To number_of_autocorrect_words-1
            autocorrect_words$(f-1)=autocorrect_words$(f)
            autocorrect_words_selected(f-1)=autocorrect_words_selected(f)
          Next f
          ReDim autocorrect_words$(ArraySize(autocorrect_words$())-1)   
          ReDim autocorrect_words_selected(ArraySize(autocorrect_words_selected())-1) 
          flag=#True
        EndIf
        
        number_of_autocorrect_words-1
        update_number_autocorrect_words_selected_count()
        SetGadgetAttribute(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_autocorrect_words-1)
        selected_autocorrect_word=-1 ; Reset the current word selected
        
        autocorrect_changed=#True
        update_number_autocorrect_words()
        global_update_virtual_vertical_scroll_bar(-1)
        
        ProcedureReturn #True
        
      EndIf
    EndIf
    
    ProcedureReturn #False
    
  EndProcedure
  
  
  
  
  ; Erase words in the Autocorrect List
  ; V1.0 - 7/JAN/2016
  ; V1.1 - 26/DEC/2016
  ;        o Started implementing the multiselection.  
  ; V1.2 - 8/JUN/2017
  ;        o undo/redo support    
  ; V1.3 - 1/DEC/2017
  ;        o Message Requesters now have a sign.  
  ;
  Procedure erase_autocorrect()
    If number_of_autocorrect_words=0
      MessageRequester("Warning!","No autocorrect words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf
    
    ; Confirm if to erase the Autocorrect List if the file in memory has been changed
    If autocorrect_changed=#True
      If MessageRequester("Warning!","The Autocorrect List has been changed."+#LF$+#LF$+"Continue?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Info)=#PB_MessageRequester_No
        ProcedureReturn #False
      EndIf
    EndIf     
    
    If number_of_autocorrect_words>0
      If MessageRequester("Warning!","Really delete all autocorrect words?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        
        autocorrect_store_undo(#True) ; 8/JUN/2017
        
        FUCKTIME=ElapsedMilliseconds()
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Erasing...")
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
        number_of_autocorrect_words=0
        ClearGadgetItems(#PANEL_BOX_AUTOCORRECT)
        Dim autocorrect_words$(0)
        Dim autocorrect_words_selected(0)
        number_of_autocorrect_words_total_selected=0
        update_number_autocorrect_words()
        autocorrect_words_filename$=""
        update_filename_autocorrect_list()
        
        selected_autocorrect_word=-1 ; Reset the current autoreplace word selected
        autocorrect_changed=#False ; Autocorrect List set to unchanged
        update_number_autocorrect_words()
        global_update_virtual_vertical_scroll_bar(-1)
        Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
        
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,100,#PB_StatusBar_Raised,0,100)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK.")
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,100) ; Reset progress bar - 21/MAR/2014
      EndIf
    EndIf
        
  EndProcedure   
  
  
  
  
  ; Sort the Autocorrect list
  ;
  ; V1.0 - 7/JAN/2016
  ; V1.1 - 26/DEC/2016
  ;        o Started implementing the multiselection.
  ; V1.2 - 08/JUN/2017
  ;        o undo/redo support    
  ; V1.3 - 01/DEC/2017
  ;        o Message Requesters now have a sign.  
  ; V1.4 - 13/NOV/2019
  ;        o Support for Latin sorting
  ; V1.5 - 02/DEC/2021
  ;        o Started showing the time taken in timely operations.
  ;
  Procedure sort_autocorrect()
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting Autocorrect List...")
    
    If number_of_autocorrect_words=0
      MessageRequester("Warning!","No autocorrect words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf
    
    autocorrect_store_undo(#True) ; 8/JUN/2017
    
    ; Show the progress bar
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,number_of_autocorrect_words*3) 
    
    ; Autocorrect List set to changed
    selected_autocorrect_word=-1 ; Reset the current word selected
    autocorrect_changed=#True
    update_number_autocorrect_words()
    
    
    time_taken=ElapsedMilliseconds()
    AddWindowTimer(#WINDOW_MAIN,800,800)
    
    
    ; Remove accents from entries
    ReDim sort_arrays_in_latin(number_of_autocorrect_words-1)
    For f=0 To number_of_autocorrect_words-1
      
      sort_arrays_in_latin(f)\in_latin=RemoveAccentsString(autocorrect_words$(f))
      sort_arrays_in_latin(f)\in_ascii=autocorrect_words$(f)
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f+1,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Removing accents... <ESC> to abort. Found:"+FormatNumber(f+1,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      EndIf       
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf      
      
    Next f
    
    
    ; Sort the array and delete the list 
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting... <ESC> to abort. Found:"+FormatNumber(number_of_autocorrect_words,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    SortStructuredArray(sort_arrays_in_latin(),#PB_Sort_Ascending,OffsetOf(sort_arrays_in_latin\in_latin.s),TypeOf(sort_arrays_in_latin\in_latin.s))
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,number_of_autocorrect_words*2,#PB_StatusBar_Raised)    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Sorting... <ESC> to abort. Found:"+FormatNumber(number_of_autocorrect_words,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    
    
    ; Populate the list with the sorted words
    For f=0 To number_of_autocorrect_words-1
      
      autocorrect_words$(f)=sort_arrays_in_latin(f)\in_ascii
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,(number_of_autocorrect_words*2)+f+1,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Populating with accents... <ESC> to abort. Found:"+FormatNumber(f+1,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      EndIf       
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf      
      
    Next f    
    
    
    ; Populate the ListIconGadget
    ClearGadgetItems(#PANEL_BOX_AUTOCORRECT)       
    SetGadgetState(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,0)
    global_update_virtual_vertical_scroll_bar(-1)
    
    
    ; Clear progress bar
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,1) ; Reset progress bar - 21/MAR/2014
    
    
    ; No selected items
    update_number_autocorrect_words_selected_none()
    
    
    RemoveWindowTimer(#WINDOW_MAIN,800)    
    
  EndProcedure  
  
  
  
  
  ; To always have the current autocorrect word highlighted in the listicongadget
  ; V1.0 - 7/JAN/2016
  Procedure update_vertical_scroll_bar_autocorrect_list_current_item()  
    If selected_autocorrect_word=-1 And GetGadgetState(#PANEL_BOX_AUTOCORRECT)=-1 : ProcedureReturn #False : EndIf
  
    selected_autocorrect_word=GetGadgetState(#PANEL_BOX_AUTOCORRECT)+GetGadgetState(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR)   
    t=GetGadgetState(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR)
    t1=selected_autocorrect_word-t
    
  EndProcedure
  
  
  
  
; Extract the Autocorrect wordlist as .txt
;
; V1.0 - 17/FEB/2016
; V1.1 - 1/DEC/2017
;        o Message Requesters now have a sign.
; V1.2 - 15/FEB/2020
;        o All file operations support BOM.
; V1.3 - 21/FEB/2020
;        o All remaining export operations now use UNIX #LF$
; V1.4 - 20/NOV/2020
;        02/DEC/2020
;        o Extracting wordlists now shows the filesize of the target file.
; V1.5 - 02/DEC/2021
;        o Started showing the time taken in timely operations.
  
;
Procedure extract_autocorrect()

    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Extracting Autocorrect List... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) 
    
    If number_of_autocorrect_words=0
      MessageRequester("Warning!","No autocorrect words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf
    
    
    file$=SaveFileRequester("Extract Autocorrect List to file", "", "Text Documents (*.txt)|*.txt", 0)
    If file$=""
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf
    If LCase(Right(file$,4))<>".txt" : file$+".txt": EndIf
    
    

    If FileSize(file$)>0
      If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        ProcedureReturn #False
      EndIf      
    EndIf    
    
    DeleteFile(file$) ; Delete .TXT file
                 

    ; Saving the .TXT file
    AddWindowTimer(#WINDOW_MAIN,800,800)
    OpenFile(1,file$)
      WriteStringFormat(1,#PB_UTF8)
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_autocorrect_words)     
      time_taken=ElapsedMilliseconds()
      For f=0 To number_of_autocorrect_words-1
        text$=autocorrect_words$(f)
        text$=ReplaceString(text$,#LF$," "+Chr($2192)+" ")
        WriteString(1,text$+#LF$,#PB_UTF8)
                      
        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event = WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Extracting Autocorrects... <ESC> to abort. Found:"+FormatNumber(f+1,0,".",number_separator_chr$)+" "+ShowFileSizeFormatted("",Lof(1),#True)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
        EndIf 
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
          If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            CloseFile(1)
            RemoveWindowTimer(#WINDOW_MAIN,800)
            ProcedureReturn #False
         EndIf
        EndIf
        

      Next f
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Extracting Autocorrects... <ESC> to abort. Found:"+FormatNumber(f+1,0,".",number_separator_chr$)+" ("+StrF(Lof(1)/1024/1024,1)+" MB)"+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      RemoveWindowTimer(#WINDOW_MAIN,800)      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      filesize_lof=Lof(1)
      CloseFile(1)
      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK. "+ShowFileSizeFormatted("",filesize_lof,#True)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))

EndProcedure




; This procedure will convert hex symbols from the autocorrect to characters
; Example: "&#x2755;"
;
; V1.0 - 18/FEB/2016
; V1.1 - 1/MAR/2016
;        o It now warns if an hex value is above 65535 which is the maximum PB accepts
; V1.2 - 8/JUN/2017
;        o undo/redo support 
; V1.3 - 25/OCT/2017
;        o Formats the total number of items to be better seen.
; V1.4 - 1/DEC/2017
;        o Message Requesters now have a sign.
; V1.5 - 02/DEC/2021
;        o Started showing the time taken in timely operations.
;
Procedure cleanup_symbols_autocorrect()
  
    Debug "Pressed CLEAN UP HEX SYMBOLS AUTOCORRECT"
  
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Clean hex symbols... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_autocorrect_words=0
      MessageRequester("Warning!","No Autocorrects found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf    
    
    
    autocorrect_store_undo(#True) ; 8/JUN/2017
      
    
    ; Processing wordlist - STEP 1 of 2
    AddWindowTimer(#WINDOW_MAIN,800,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_autocorrect_words)     
    time_taken=ElapsedMilliseconds()
      
    
    symbol_above_65535=#False ; To ask if people want to abort the conversion, since PB is limited to 65535 - 1/MAR/2016    
    word_fixes=0
    For f=0 To number_of_autocorrect_words-1
      text$=autocorrect_words$(f)
      
      null=#False ; To avoid infinite loop
      Repeat
        If FindString(text$,"&#x")>0
          t$=""
          For ff=FindString(text$,"&#x") To Len(text$)
            If Mid(text$,ff,1)<>";" : t$+Mid(text$,ff,1) : EndIf
            If Mid(text$,ff,1)=";" : t$+";" : Break : EndIf
          Next ff
          If t$<>"" And Left(t$,3)="&#x" And Right(t$,1)=";"
            text$=ReplaceString(text$,t$,Chr(Val("$"+Mid(t$,4,Len(t$)-4))))
            If Val("$"+Mid(t$,4,Len(t$)-4))>65535 And symbol_above_65535=#False : symbol_above_65535=#True : EndIf
          Else
            null=#True            
          EndIf
        EndIf
      Until FindString(text$,"&#x")=0 Or null=#True
      
      ; Abort if an invalid word is found
      If null=#True
        Debug "INVALID WORD FOUND!"          
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Null word at Line:"+FormatNumber(f+1,0,".",number_separator_chr$)+" Aborted.")
        RemoveWindowTimer(#WINDOW_MAIN,800)
        ProcedureReturn #False        
      EndIf
      
      
      ; Increment fixes
      If text$<>autocorrect_words$(f)
        word_fixes+1
      EndIf      
      
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Processing Wordlist..."+FormatNumber(f,0,".",number_separator_chr$)+" <ESC> To abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      EndIf 
      
      
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)          
          ProcedureReturn #False
       EndIf
      EndIf
        
    Next f
    
    
    ; No fixes found
    If word_fixes=0
      StatusBarText(#WINDOW_MAIN_STATUS_BAR,0,"OK."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
      MessageRequester("Warning!","No hex symbols found!",#PB_MessageRequester_Warning)      
      RemoveWindowTimer(#WINDOW_MAIN,800)
      ProcedureReturn #False      
    EndIf
    

    ; Ask if user wants to abort conversion if found a symbol above 65535
    If symbol_above_65535=#True
      If MessageRequester("Warning!","Found symbols above character 65535."+#LF$+"(PureBasic limitation - character corruption)"+#LF$+#LF$+"Abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
        RemoveWindowTimer(#WINDOW_MAIN,800)          
        ProcedureReturn #False
      EndIf      
    EndIf
            
    
    ; Converting wordlist - STEP 2 of 2
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_autocorrect_words)     
    
    autocorrect_changed=#True ; Autocorrect Changed=#True
    
    For f=0 To number_of_autocorrect_words-1
      text$=autocorrect_words$(f)
            
      Repeat
        If FindString(text$,"&#x")>0
          t$=""
          For ff=FindString(text$,"&#x") To Len(text$)
            If Mid(text$,ff,1)<>";" : t$+Mid(text$,ff,1) : EndIf
            If Mid(text$,ff,1)=";" : t$+";" : Break : EndIf
          Next ff
          If t$<>"" And Left(t$,3)="&#x" And Right(t$,1)=";"        
            text$=ReplaceString(text$,t$,Chr(Val("$"+Mid(t$,4,Len(t$)-4))))
          EndIf
        EndIf
      Until FindString(text$,"&#x")=0

      
      ; Change in the original word
      autocorrect_words$(f)=text$
      
    
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Converting hex symbols..."+FormatNumber(f,0,".",number_separator_chr$)+" <ESC> To abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)          
          ProcedureReturn #False
       EndIf
      EndIf
        
    Next f
  
    
    RemoveWindowTimer(#WINDOW_MAIN,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK. Converted symbols:"+FormatNumber(word_fixes,0,".",number_separator_chr$)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))  
    global_update_virtual_vertical_scroll_bar(-1) 
   
EndProcedure




; To find duplicates in the Autocorrect wordlist
;
; V1.0 -  18/FEB/2016
; V1.1 -  9/SEP/2016
;         o Fixed the scroll bar size after a delete duplicates
; V1.2 - 26/DEC/2016
;        o Started implementing the multiselection.  
; V1.3 - 11/MAR/2017
;        o Changed the buttons height to 25. 
; V1.4 - 15/APR/2017
;        o Added exclusive delete to autocorrect.  
; V1.5 - 8/JUN/2017
;        o undo/redo support  
; V1.6 - 25/OCT/2017
;        o Formats the total number of items to be better seen.
; V1.7 - 1/DEC/2017
;        o Message Requesters now have a sign. 
; V1.8 - 06/JUN/2019
;        o GTK3 Ready
; V1.9 - 20/JUN/2019
;        o Preferences allow to choose the font for the language.
; V2.0 - 15/FEB/2020
;        o All file operations support BOM.
; V2.1 - 21/FEB/2020
;        o All remaining export operations now use UNIX #LF$
;
Procedure show_duplicates_autocorrect()
  
    Debug "Pressed SHOW/DELETE DUPLICATES AUTOCORRECT"
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 1/2: Extracting... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_autocorrect_words=0
      MessageRequester("Warning!","No autocorrects found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
      
            
    ; Step 1 of 2 (extracting autocorrect wordlist)
    AddWindowTimer(#WINDOW_MAIN,800,800)
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_autocorrect_words)     
    time=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
    Dim total_number_of_processed_words_array$(number_of_autocorrect_words-1)
    
    
    For f=0 To number_of_autocorrect_words-1
      
      ; Copy the Autocorrect list to the words processed array
;       text$=StringField(autocorrect_words$(f),1,#LF$)
      text$=autocorrect_words$(f)
      total_number_of_processed_words_array$(f)=text$
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 1/2: Extracting..."+FormatNumber(f,0,".",number_separator_chr$)+" <ESC> to abort.")
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f
    
    
    ; Sort the array    
    SortArray(total_number_of_processed_words_array$(),#PB_Sort_Ascending)    
    
    
    ; Step 2 of 2 (comparing autocorrect wordlist)    
    
    ; First open a hidden window
    If OpenWindow(#WINDOW_SHOW_DUPLICATES,0,0,400+20-5+200-60-30-20-15-15-5-5,10+-4-2-1+10-3+40-100+200-50-20-20+100+100+200-30-30-30+10-20-10,"Show Duplicates",#PB_Window_Invisible|#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=#False : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    ListIconGadget(#LISTICON_WINDOW_SHOW_DUPLICATES,5,5,2+3+400+30+10+4+3+1,200+100-50+10,"Word",100+100,#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection)
    AddGadgetColumn(#LISTICON_WINDOW_SHOW_DUPLICATES,1,"Occurrences",100)
    SetGadgetFont(#LISTICON_WINDOW_SHOW_DUPLICATES,FontID(#font_for_language_id))
    ButtonGadget(#BUTTON_WINDOW_SHOW_DELETE,5+50+50+50+0+50-50,200+100-50+10+10,100,default_gadget_height_button,"Delete")
    ButtonGadget(#BUTTON_WINDOW_SHOW_XDELETE,5+50+50+50+0+50-50,5+200+100-50+10+10+30+3,100,default_gadget_height_button,"XDelete")
    ButtonGadget(#BUTTON_WINDOW_SHOW_EXPORT,5+50+50+50+0+50+60,200+100-50+10+10,100,default_gadget_height_button,"Export")
    ButtonGadget(#BUTTON_WINDOW_SHOW_OKAY,5+50+50+50+50+0+100+10+60,200+100-50+10+10,100-10-10,default_gadget_height_button,"Close")
    
    ; Progress bar
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 2/2: Comparing... <ESC> to abort.")    
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,number_of_autocorrect_words)     
        
    
    total_occurrences=0 ; To be used in the progress bar of delete duplicates
    For f=0 To number_of_autocorrect_words-1
      t$=StringField(total_number_of_processed_words_array$(f),1,#LF$)
      occurrences=0
      
      For ff=f To number_of_autocorrect_words-1            
                
        
        If t$=StringField(total_number_of_processed_words_array$(ff),1,#LF$)
          occurrences+1          
          total_occurrences+1
        Else
          Break
        EndIf
                             
      Next ff
      
      f=ff-1
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
         StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Show duplicates. 2/2: Comparing..."+FormatNumber(f,0,".",number_separator_chr$)+"/"+Str(number_of_autocorrect_words)+" <ESC> to abort.")    
         StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          CloseWindow(#WINDOW_SHOW_DUPLICATES)
          ProcedureReturn #False
        EndIf
      EndIf      
      
      
      ; Add duplicate word to listicongadget
      If occurrences>1
          AddGadgetItem(#LISTICON_WINDOW_SHOW_DUPLICATES,-1,StringField(t$,1,#LF$)+#LF$+Str(occurrences))
      EndIf
      
    Next f    
    
    
    Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
    RemoveWindowTimer(#WINDOW_MAIN,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
   
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK.")
    
    
    ; No duplicates found, exit the Procedure - 7/AUG/2015
    If CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)=0
      MessageRequester("Warning!","No duplicates found.",#PB_MessageRequester_Warning)
      DisableWindow(#WINDOW_MAIN,#False)
      CloseWindow(#WINDOW_SHOW_DUPLICATES)                
      ProcedureReturn
    EndIf    
    
    
    HideWindow(#WINDOW_SHOW_DUPLICATES,#False)
    DisableWindow(#WINDOW_MAIN,#True)
    SetActiveWindow(#WINDOW_SHOW_DUPLICATES)
    
    ; Add the number of words found
    TextGadget(#TEXT_WINDOW_SHOW_DUPLICATES_FOUND,5+5,200+100-50+10+10,200-20,default_gadget_height_text,"Found:"+FormatNumber(CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES),0,".",number_separator_chr$))
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_SHOW_DUPLICATES,#PB_Shortcut_Escape,1000)
    
    cancel=#False
    Repeat
      
      event=WaitWindowEvent()
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        cancel=#True
      EndIf
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf
      
      Select event 
        Case #PB_Event_Gadget
         Select EventGadget()
           Case #BUTTON_WINDOW_SHOW_OKAY
             cancel=#True
             
             
           Case #BUTTON_WINDOW_SHOW_DELETE
             If CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)>0
               If MessageRequester("Warning!","Really delete duplicates?"+#LF$+"(notice this will sort the array)",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes       
                 
                  autocorrect_store_undo(#True) ; 8/JUN/2017
                 
                  ; Progress bar
                  StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Removing duplicates... 0/"+FormatNumber(total_occurrences,0,".",number_separator_chr$))
                  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,total_occurrences)             
             
                  ; # of fixed (deleted) words
                  fixed_words=0
             
                  ; Remove duplicates
                  AddWindowTimer(#WINDOW_SHOW_DUPLICATES,800,800)
                  For f=0 To number_of_autocorrect_words-1-1
               
                    If StringField(total_number_of_processed_words_array$(f),1,#LF$)=StringField(total_number_of_processed_words_array$(f+1),1,#LF$)
                      total_number_of_processed_words_array$(f)=""
                      fixed_words+1
                    EndIf
               
                    ; Fix to window stop responding if we click on it while deleting duplicates
                    event = WindowEvent()
                    If event=#PB_Event_Timer And EventTimer()=800
                      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,fixed_words,#PB_StatusBar_Raised)
                      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Removing duplicates... "+FormatNumber(fixed_words,0,".",number_separator_chr$)+"/"+FormatNumber(total_occurrences,0,".",number_separator_chr$))
                    EndIf               

                  Next f
                  RemoveWindowTimer(#WINDOW_SHOW_DUPLICATES,800)
             
             
                  ; Populate list
                  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
                  StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Populating list... "+"0/"+FormatNumber(number_of_autocorrect_words-fixed_words,0,".",number_separator_chr$))
                  
                  AddWindowTimer(#WINDOW_SHOW_DUPLICATES,800,800)
                  f=0
                  For ff=0 To number_of_autocorrect_words-1

                    If total_number_of_processed_words_array$(ff)<>""
                      autocorrect_words$(f)=total_number_of_processed_words_array$(ff)
                      f+1
                    EndIf
               
                    ; Fix to window stop responding if we click on it while deleting duplicates
                    event = WindowEvent()
                    If event=#PB_Event_Timer And EventTimer()=800
                      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,ff,#PB_StatusBar_Raised)
                      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Populating list... "+FormatNumber(f,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_autocorrect_words-fixed_words,0,".",number_separator_chr$))
                    EndIf               

                  Next ff
                  RemoveWindowTimer(#WINDOW_SHOW_DUPLICATES,800)
             
                  ; Redim the array
                  number_of_autocorrect_words-fixed_words
                  ReDim autocorrect_words$(number_of_autocorrect_words-1)
                  ReDim autocorrect_words_selected(number_of_autocorrect_words-1)
                  update_number_autocorrect_words_selected_none()
                  
                  ; Fixed the scroll bar size after a delete duplicates - 9/SEP/2016
                  SetGadgetAttribute(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_autocorrect_words-1)
                  selected_autocorrect_word=-1 ; Reset the current word selected    
             
                  ; Refresh the ListIconGadget
                  global_update_virtual_vertical_scroll_bar(-1)
             
                  ; Thesaurus=Changed
                  autocorrect_changed=#True
             
                  ; OK in status bar + reset it
                  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar
                  StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK.")
                  
                  ; Close the Duplicates window and activate the main window
                  DisableWindow(#WINDOW_MAIN,#False)
                  CloseWindow(#WINDOW_SHOW_DUPLICATES)
                  ProcedureReturn #True            
                 
                 
               EndIf
             EndIf  
             
             
             
           Case #BUTTON_WINDOW_SHOW_XDELETE
             If CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)>0
               If MessageRequester("Warning!","Really eXclusive delete duplicates?"+#LF$+"(notice this will sort the array)",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes       
                 
                  autocorrect_store_undo(#True) ; 8/JUN/2017
                 
                  ; Progress bar
                  StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Removing duplicates... 0/"+FormatNumber(total_occurrences,0,".",number_separator_chr$))
                  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,total_occurrences)             
             
                  ; # of fixed (deleted) words
                  fixed_words=0
             
                  ; Remove duplicates
                  AddWindowTimer(#WINDOW_SHOW_DUPLICATES,800,800)
                  For f=0 To number_of_autocorrect_words-1-1
               
;                     If StringField(total_number_of_processed_words_array$(f)+#LF$,1,#LF$)=StringField(total_number_of_processed_words_array$(f+1)+#LF$,1,#LF$)
;                       total_number_of_processed_words_array$(f)=""
;                       total_number_of_processed_words_array$(f+1)=""
;                       fixed_words+2
;                     EndIf
                    
                    If StringField(total_number_of_processed_words_array$(f),1,#LF$)=StringField(total_number_of_processed_words_array$(f+1),1,#LF$)
                      t$=StringField(total_number_of_processed_words_array$(f),1,#LF$)
                      If f+1<=number_of_autocorrect_words-1
                        For ff=f To number_of_autocorrect_words-1
                          If StringField(total_number_of_processed_words_array$(ff),1,#LF$)=t$
                            total_number_of_processed_words_array$(ff)=""
                            fixed_words+1
                            f=ff
                          Else
                            Break
                          EndIf
                        Next ff
                      EndIf  
                    EndIf                    
               
                    ; Fix to window stop responding if we click on it while deleting duplicates
                    event = WindowEvent()
                    If event=#PB_Event_Timer And EventTimer()=800
                      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,fixed_words,#PB_StatusBar_Raised)
                      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Removing duplicates... "+FormatNumber(fixed_words,0,".",number_separator_chr$)+"/"+FormatNumber(total_occurrences,0,".",number_separator_chr$))
                    EndIf               

                  Next f
                  RemoveWindowTimer(#WINDOW_SHOW_DUPLICATES,800)
             
             
                  ; Populate list
                  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
                  StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Populating list... "+"0/"+FormatNumber(number_of_autocorrect_words-fixed_words,0,".",number_separator_chr$))
                  
                  AddWindowTimer(#WINDOW_SHOW_DUPLICATES,800,800)
                  f=0
                  For ff=0 To number_of_autocorrect_words-1

                    If total_number_of_processed_words_array$(ff)<>""
                      autocorrect_words$(f)=total_number_of_processed_words_array$(ff)
                      f+1
                    EndIf
               
                    ; Fix to window stop responding if we click on it while deleting duplicates
                    event = WindowEvent()
                    If event=#PB_Event_Timer And EventTimer()=800
                      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,ff,#PB_StatusBar_Raised)
                      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Populating list... "+FormatNumber(f,0,".",number_separator_chr$)+"/"+FormatNumber(number_of_autocorrect_words-fixed_words,0,".",number_separator_chr$))
                    EndIf               

                  Next ff
                  RemoveWindowTimer(#WINDOW_SHOW_DUPLICATES,800)
             
                  ; Redim the array
                  number_of_autocorrect_words-fixed_words
                  If number_of_autocorrect_words-1>0
                    ReDim autocorrect_words$(number_of_autocorrect_words-1)
                    ReDim autocorrect_words_selected(number_of_autocorrect_words-1)
                    update_number_autocorrect_words_selected_none()                    
                  Else
                    ReDim autocorrect_words$(0)
                    ReDim autocorrect_words_selected(0)
                    autocorrect_words_selected(0)=#False
                    number_of_autocorrect_words_total_selected=0
                    global_update_virtual_vertical_scroll_bar(-1)                    
                  EndIf                    
;                   update_number_autocorrect_words_selected_none()
                  
                  ; Fixed the scroll bar size after a delete duplicates - 9/SEP/2016
                  SetGadgetAttribute(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_autocorrect_words-1)
                  selected_autocorrect_word=-1 ; Reset the current word selected    
             
                  ; Refresh the ListIconGadget
                  global_update_virtual_vertical_scroll_bar(-1)
             
                  ; Autocorrect=Changed
                  autocorrect_changed=#True
             
                  ; OK in status bar + reset it
                  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar
                  StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK.")
                  
                  ; Close the Duplicates window and activate the main window
                  DisableWindow(#WINDOW_MAIN,#False)
                  CloseWindow(#WINDOW_SHOW_DUPLICATES)
                  ProcedureReturn #True            
                 
                 
               EndIf
             EndIf  
             
             
             
           Case #BUTTON_WINDOW_SHOW_EXPORT
             If CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)>0
               flag=#False
               file$=SaveFileRequester("Export Duplicates to file","","Text Documents (*.txt)|*.txt",0)
               If file$<>""
                 If LCase(Right(file$,4))<>".txt" : file$+".txt": EndIf
                 
                 t=ReadFile(1,file$)
                 If t<>0
                   CloseFile(1)
                   If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                       DeleteFile(file$) ; Delete .TXT file                 
                   Else
                       flag=#True ; Abort the save as the file already exists and pressed to not overwrite it
                   EndIf  
                 EndIf    
                 
                 If flag=#False
                    ; Saving the .TXT file
                    AddWindowTimer(#WINDOW_MAIN,800,800)
                    OpenFile(1,file$)
                    WriteStringFormat(1,#PB_UTF8)
                    
                    WriteString(1,"DUPLICATES FOUND:"+FormatNumber(CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES),0,".",number_separator_chr$)+#LF$,#PB_UTF8)
                    WriteString(1,""+#LF$,#PB_UTF8)
                    
                    For f=1 To CountGadgetItems(#LISTICON_WINDOW_SHOW_DUPLICATES)
                      WriteString(1,GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,0)+":"+GetGadgetItemText(#LISTICON_WINDOW_SHOW_DUPLICATES,f-1,1)+#LF$,#PB_UTF8)
                      
                      ; Fix to window stop responding if we click on it while exporting
                      event = WindowEvent()
                      If event=#PB_Event_Timer And EventTimer()=800 : EndIf 
                
                    Next f
                    CloseFile(1)
                    RemoveWindowTimer(#WINDOW_MAIN,800)          
                 EndIf
                 
               EndIf
               
             EndIf
         EndSelect
         
      EndSelect   
      
    Until cancel=#True
    
    ; Close the Duplicates window and activate the main window
    DisableWindow(#WINDOW_MAIN,#False)
    CloseWindow(#WINDOW_SHOW_DUPLICATES)    
    
EndProcedure




; Fix spaces in Autocorrect
;
; V1.0 - 18/FEB/2016
; V1.1 - 8/JUN/2017
;        o undo/redo support  
; V1.2 - 25/OCT/2017
;        o Formats the total number of items to be better seen.
; V1.3 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure fix_spaces_autocorrect()
  
    Debug "Pressed FIX SPACES AUTOCORRECT"
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fix Invalid Spaces... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)    
    
    
    If number_of_autocorrect_words=0
      MessageRequester("Warning!","No Autocorrects found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
    
    
    autocorrect_store_undo(#True) ; 8/JUN/2017
    
    ; Processing wordlist - STEP 1 of 2
    AddWindowTimer(#WINDOW_MAIN,800,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_autocorrect_words)     
    time=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
      
    
    word_fixes=0
    For f=0 To number_of_autocorrect_words-1
      text$=autocorrect_words$(f)
      
      Repeat
        text$=ReplaceString(text$,"  "," ")
        text$=ReplaceString(text$," "+#LF$,#LF$)
        text$=ReplaceString(text$,#LF$+" ",#LF$)
        If Left(text$,1)=" " : text$=Mid(text$,2) : EndIf
        If Right(text$,1)=" " : text$=Left(text$,Len(text$)-1) : EndIf
      Until FindString(text$,"  ")=0 And FindString(text$," "+#LF$)=0 And FindString(text$,#LF$+" ")=0  And Left(text$,1)<>" " And Right(text$,1)<>" "
      
      ; Abort if an empty word is found
      If StringField(text$,1,#LF$)="" Or StringField(text$,2,#LF$)=""
        Debug "NULL WORD FOUND!"          
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Null word at Line:"+Str(f+1)+" Aborted.")
        RemoveWindowTimer(#WINDOW_MAIN,800)
        ProcedureReturn #False        
      EndIf
      
      
      ; Increment fixes
      If text$<>autocorrect_words$(f)
        word_fixes+1
      EndIf      
      
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Processing Wordlist 1/2..."+FormatNumber(f,0,".",number_separator_chr$)+" <ESC> To abort.")
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)          
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f
    
    
    ; No fixes found
    If word_fixes=0
      MessageRequester("Warning!","No spaces to fix found!",#PB_MessageRequester_Warning)
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK.")
      RemoveWindowTimer(#WINDOW_MAIN,800)
      ProcedureReturn #False      
    EndIf
    
    
    
    ; Fixing spaces - STEP 2 of 2    
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fixing spaces 2/2...0 <ESC> To abort.")
    
    word_fixes=0
    For f=0 To number_of_autocorrect_words-1
      text$=autocorrect_words$(f)
      
      Repeat
        text$=ReplaceString(text$,"  "," ")
        text$=ReplaceString(text$," "+#LF$,#LF$)
        text$=ReplaceString(text$,#LF$+" ",#LF$)
        If Left(text$,1)=" " : text$=Mid(text$,2) : EndIf
        If Right(text$,1)=" " : text$=Left(text$,Len(text$)-1) : EndIf
      Until FindString(text$,"  ")=0 And FindString(text$," "+#LF$)=0 And FindString(text$,#LF$+" ")=0  And Left(text$,1)<>" " And Right(text$,1)<>" "
      
      ; Increment fixes + fix directly in the array
      If text$<>autocorrect_words$(f)
        word_fixes+1
        autocorrect_changed=#True
        autocorrect_words$(f)=text$
      EndIf

      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fixing spaces 2/2..."+FormatNumber(f,0,".",number_separator_chr$)+" <ESC> To abort.")
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          RemoveWindowTimer(#WINDOW_MAIN,800)
          global_update_virtual_vertical_scroll_bar(-1)
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f    
   
    
    Debug "Time taken:"+Str((ElapsedMilliseconds()-FUCKTIME)/1000)
    RemoveWindowTimer(#WINDOW_MAIN,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Fixed words:"+FormatNumber(word_fixes,0,".",number_separator_chr$))
    global_update_virtual_vertical_scroll_bar(-1)
    
EndProcedure




; To show the total of items selected
; 
; V1.0 - 22/DEC/2016
;        26/DEC/2016
;
Procedure update_number_autocorrect_words_selected()
  
  t$="Selected:"
  If number_of_autocorrect_words_total_selected=0 : t$+"None" : EndIf
  If number_of_autocorrect_words_total_selected<>0 And number_of_autocorrect_words_total_selected<>number_of_autocorrect_words : t$+Str(number_of_autocorrect_words_total_selected) : EndIf
  If number_of_autocorrect_words_total_selected=number_of_autocorrect_words And number_of_autocorrect_words<>0 : t$+"All" : EndIf
  
  If GetGadgetText(#TEXT_PANEL_BOX_AUTOCORRECT_NUMBER_WORDS_SELECTED)<>t$ : SetGadgetText(#TEXT_PANEL_BOX_AUTOCORRECT_NUMBER_WORDS_SELECTED,t$) : EndIf
    
EndProcedure




; To select ALL in the array
; 
; V1.0 - 26/DEC/2016
; V1.1 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure update_number_autocorrect_words_selected_all()
  
  If number_of_autocorrect_words=0
    MessageRequester("Warning!","No autocorrect words found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf
  
  If number_of_autocorrect_words>0
    For f=0 To number_of_autocorrect_words-1
      autocorrect_words_selected(f)=#True
    Next f
  EndIf
  
  number_of_autocorrect_words_total_selected=number_of_autocorrect_words
  
  global_update_virtual_vertical_scroll_bar(-1)
  
EndProcedure




; To select NONE in the array
; 
; V1.0 - 26/DEC/2016
; V1.1 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure update_number_autocorrect_words_selected_none()
  
  If number_of_autocorrect_words=0
    MessageRequester("Warning!","No autocorrect words found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf  
  
  If number_of_autocorrect_words>0
    For f=0 To number_of_autocorrect_words-1
      autocorrect_words_selected(f)=#False
    Next f
  EndIf  
  
  number_of_autocorrect_words_total_selected=0
  
  global_update_virtual_vertical_scroll_bar(-1)
  
EndProcedure




; To INVERT in the array
; 
; V1.0 - 2/MAR/2018
;   
Procedure update_number_autocorrect_words_selected_invert()
  
  If number_of_autocorrect_words=0
    MessageRequester("Warning!","No autocorrect words found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf  
  
  count=0
  If number_of_autocorrect_words>0
    For f=0 To number_of_autocorrect_words-1
      v=autocorrect_words_selected(f)
      If v=#False
        v=#True
        count+1
      Else
        v=#False
      EndIf      
      autocorrect_words_selected(f)=v
    Next f
  EndIf  
  
  number_of_autocorrect_words_total_selected=count
  
  global_update_virtual_vertical_scroll_bar(-1)
  
EndProcedure




; Count the number of selected items after a delete and such
;
; V1.0 - 26/DEC/2016
;
Procedure update_number_autocorrect_words_selected_count()
  
  number_of_autocorrect_words_total_selected=0
  
  If number_of_autocorrect_words>0
    For f=0 To number_of_autocorrect_words-1
      If autocorrect_words_selected(f)=#True : number_of_autocorrect_words_total_selected+1 : EndIf
    Next f
  EndIf    
  
EndProcedure




; Delete the number of selected items
;
; V1.0 - 26/DEC/2016
; V1.1 - 1/JAN/2017
;        o Fix deleting selected only when there was one item
; V1.2 - 8/JUN/2017
;        o undo/redo support  
; V1.3 - 1/DEC/2017
;        o Message Requesters now have a sign. 
;
Procedure update_number_autocorrect_words_selected_delete()
  
  
  If number_of_autocorrect_words=0
    MessageRequester("Warning!","No autocorrect words found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf
  
  
  If number_of_autocorrect_words_total_selected=0
    MessageRequester("Warning!","No selected autocorrect words found.",#PB_MessageRequester_Warning)
    ProcedureReturn
  EndIf  
  
  
  t$=""
  If number_of_autocorrect_words_total_selected>1 : t$="s" : EndIf
  If MessageRequester("Warning!","Really delete "+Str(number_of_autocorrect_words_total_selected)+" autocorrect word"+t$+"?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No  
    ProcedureReturn
  EndIf
  
  autocorrect_store_undo(#False) ; 8/JUN/2017
  
  Dim buffer$(number_of_autocorrect_words-1)
  
  counter=0
  position=0
  For f=0 To number_of_autocorrect_words-1
    If autocorrect_words_selected(f)=#False
      buffer$(position)=autocorrect_words$(f)
      position+1
    EndIf
    If autocorrect_words_selected(f)=#True
      counter+1
      autocorrect_words_selected(f)=#False
    EndIf
  Next f
  
  CopyArray(buffer$(),autocorrect_words$())
  number_of_autocorrect_words-counter  
  zero_items=#False
  If number_of_autocorrect_words=0
    zero_items=#True
    number_of_autocorrect_words=1
  EndIf
  ReDim autocorrect_words$(number_of_autocorrect_words-1)
  ReDim autocorrect_words_selected(number_of_autocorrect_words-1)
  
  number_of_autocorrect_words_total_selected=0
  
  SetGadgetAttribute(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_autocorrect_words-1)
  selected_autocorrect_word=-1 ; Reset the current word selected
  
  ; Fix deleting only when there was one item
  If zero_items=#True : number_of_autocorrect_words=0 : EndIf
  
  autocorrect_changed=#True
  update_number_autocorrect_words()
  global_update_virtual_vertical_scroll_bar(-1)
  
EndProcedure




; Clone the autocorrect words
;
; V1.0 - 18/MAR/2017
; V1.1 - 8/JUN/2017
;        o undo/redo support  
; V1.2 - 1/DEC/2017
;        o Message Requesters now have a sign.  
;
Procedure clone_autocorrect()
  
    If number_of_autocorrect_words=0
      MessageRequester("Warning!","No autocorrect words found.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf    
    
    
    t=selected_autocorrect_word
    If t=-1
      MessageRequester("Warning!","No autocorrect word selected.",#PB_MessageRequester_Warning)
      ProcedureReturn #False
    EndIf
    
    autocorrect_store_undo(#False) ; 8/JUN/2017
    
    autocorrect_changed=#True
    ReDim autocorrect_words$(number_of_autocorrect_words)        
    ReDim autocorrect_words_selected(number_of_autocorrect_words) 
    autocorrect_words$(number_of_autocorrect_words)=autocorrect_words$(t)
    number_of_autocorrect_words+1
    SetGadgetAttribute(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_autocorrect_words-1)
    update_number_autocorrect_words()
    global_update_virtual_vertical_scroll_bar(-1)
    update_vertical_scroll_bar_autocorrect_list_current_item()
        
          
EndProcedure




; Disable undo + redo
;
; V1.0 - 8/JUN/2017
;
Procedure autocorrect_disable_undo_redo()
  
  DisableGadget(#BUTTON_PANEL_BOX_AUTOCORRECT_UNDO,#True)
  Dim autocorrect_undo_array(0)
  Dim autocorrect_undo_array_total_selected(0)    
  autocorrect_undo=#False
  autocorrect_undo_number_items=0
  autocorrect_undo_selected_item=-1
  autocorrect_undo_items_total_selected=0
  autocorrect_undo_top_of_listicongadget=0
  
  
  DisableGadget(#BUTTON_PANEL_BOX_AUTOCORRECT_REDO,#True)    
  Dim autocorrect_redo_array(0)
  Dim autocorrect_redo_array_total_selected(0)
  autocorrect_redo=#False
  autocorrect_redo_number_items=0
  autocorrect_redo_selected_item=-1
  autocorrect_redo_items_total_selected=0
  autocorrect_redo_top_of_listicongadget=0  
  
EndProcedure




; Store undo
;
; V1.0 - 08/JUN/2017
; V1.1 - 23/MAR/2022
;        o Disable main GUI for input/output operations
;
Procedure autocorrect_store_undo(disable_gadgets)
  Debug "STORE UNDO"
  
  ; Enable undo button and disable redo button
  If disable_gadgets=#False
    DisableGadget(#BUTTON_PANEL_BOX_AUTOCORRECT_UNDO,#False)
    DisableGadget(#BUTTON_PANEL_BOX_AUTOCORRECT_REDO,#True)
  EndIf
  
  autocorrect_undo=#True
  autocorrect_redo=#False
  
  Dim autocorrect_undo_array$(ArraySize(autocorrect_words$()))
  CopyArray(autocorrect_words$(),autocorrect_undo_array$())
  Dim autocorrect_undo_array_total_selected(ArraySize(autocorrect_words_selected()))  
  CopyArray(autocorrect_words_selected(),autocorrect_undo_array_total_selected())
  autocorrect_undo_number_items=number_of_autocorrect_words
  autocorrect_undo_selected_item=selected_autocorrect_word
  autocorrect_undo_items_total_selected=number_of_autocorrect_words_total_selected
  autocorrect_undo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR)
  
EndProcedure




; Restore undo
;
; V1.0 - 8/JUN/2017
;
Procedure autocorrect_restore_undo()
  Debug "RESTORE UNDO"
  
  ; Disable undo button and enable redo button
  DisableGadget(#BUTTON_PANEL_BOX_AUTOCORRECT_UNDO,#True)
  DisableGadget(#BUTTON_PANEL_BOX_AUTOCORRECT_REDO,#False)    
  
  autocorrect_undo=#False
  autocorrect_redo=#True
  
  Dim autocorrect_words$(ArraySize(autocorrect_undo_array$()))
  CopyArray(autocorrect_undo_array$(),autocorrect_words$())  
  Dim autocorrect_words_selected(ArraySize(autocorrect_undo_array_total_selected()))
  CopyArray(autocorrect_undo_array_total_selected(),autocorrect_words_selected())
  number_of_autocorrect_words=autocorrect_undo_number_items
  selected_autocorrect_word=autocorrect_undo_selected_item
  number_of_synonyms_total_selected=autocorrect_undo_items_total_selected  
;   autocorrect_undo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR)
  
  
  ; Count number of selected words
  update_number_autocorrect_words_selected_count()
      
      
  ; Update number of selected words
  update_number_autocorrect_words_selected()
      
      
  ; Update number of words
  update_number_autocorrect_words()
      
      
  ; Reset autocorrect
  selected_autocorrect_word=-1 ; Reset the current word selected
  SetGadgetAttribute(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_autocorrect_words-1)
  global_update_virtual_vertical_scroll_bar(-1)
  
  selected_autocorrect_word=autocorrect_undo_selected_item           
      
  ; Place back the first item on top of the ListIconGadget
  SetGadgetState(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,autocorrect_undo_top_of_listicongadget)
  global_update_virtual_vertical_scroll_bar(-1)
 
  
EndProcedure




; Store redo
;
; V1.0 - 8/JUN/2017
;
Procedure autocorrect_store_redo()
  Debug "STORE REDO"
  
  ; Enable redo button and disable undo button
  DisableGadget(#BUTTON_PANEL_BOX_AUTOCORRECT_UNDO,#True) 
  DisableGadget(#BUTTON_PANEL_BOX_AUTOCORRECT_REDO,#False)
  
  autocorrect_undo=#False 
  autocorrect_redo=#True 
  
  Dim autocorrect_redo_array$(ArraySize(autocorrect_words$()))
  CopyArray(autocorrect_words$(),autocorrect_redo_array$())
  Dim autocorrect_redo_array_total_selected(ArraySize(autocorrect_words_selected()))  
  CopyArray(autocorrect_words_selected(),autocorrect_redo_array_total_selected())
  autocorrect_redo_number_items=number_of_autocorrect_words
  autocorrect_redo_selected_item=selected_autocorrect_word
  autocorrect_redo_items_total_selected=number_of_autocorrect_words_total_selected
  autocorrect_redo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR)
  
EndProcedure




; Restore redo
;
; V1.0 - 8/JUN/2017
;
Procedure autocorrect_restore_redo()
  Debug "RESTORE REDO"
  
  ; Disable redo button and enable undo button
  DisableGadget(#BUTTON_PANEL_BOX_AUTOCORRECT_UNDO,#False) 
  DisableGadget(#BUTTON_PANEL_BOX_AUTOCORRECT_REDO,#True)     
  
  autocorrect_undo=#True 
  autocorrect_redo=#False 
  
  Dim autocorrect_words$(ArraySize(autocorrect_redo_array$()))
  CopyArray(autocorrect_redo_array$(),autocorrect_words$())  
  Dim autocorrect_words_selected(ArraySize(autocorrect_redo_array_total_selected()))
  CopyArray(autocorrect_redo_array_total_selected(),autocorrect_words_selected())
  number_of_autocorrect_words=autocorrect_redo_number_items
  selected_autocorrect_word=autocorrect_redo_selected_item
  number_of_synonyms_total_selected=autocorrect_redo_items_total_selected  
;   autocorrect_redo_top_of_listicongadget=GetGadgetState(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR)
  
  
  ; Count number of selected words
  update_number_autocorrect_words_selected_count()
      
      
  ; Update number of selected words
  update_number_autocorrect_words_selected()
      
      
  ; Update number of words
  update_number_autocorrect_words()
      
      
  ; Reset autocorrect
  selected_autocorrect_word=-1 ; Reset the current word selected
  SetGadgetAttribute(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,#PB_ScrollBar_Maximum,number_of_autocorrect_words-1)
  global_update_virtual_vertical_scroll_bar(-1)
  
  selected_autocorrect_word=autocorrect_redo_selected_item           
      
  ; Place back the first item on top of the ListIconGadget
  SetGadgetState(#PANEL_BOX_AUTOCORRECT_VERTICAL_BAR,autocorrect_redo_top_of_listicongadget)
  global_update_virtual_vertical_scroll_bar(-1)
  
EndProcedure




; AC Validate
;
; To search for autocorrects not existing in the dictionary, plus invalid autocorrects
;
; V1.0 - 23/SEP/2018
;        24/SEP/2018
;        25/SEP/2018
; V1.1 - 06/DEC/2018
;        o Improved the warning message when "AC Validate" is selected and there is no dictionary.
; V1.2 - 20/JUN/2019
;        o Preferences allow to choose the font for the language. 
;        o People and Places names import now shows "<ESC> to abort." during scan.
; V1.3 - 15/FEB/2020
;        o All file operations support BOM.
; V1.4 - 04/MAY/2020
;        o AC Validate window now has a go to top+bottom button
; V1.5 - 10/NOV/2020
;        o Clear buttons now use a trashcan image.
; V1.6 - 11/NOV/2020
;        o "AC Validate" now supports GTK3
; V1.7 - 26/NOV/2020
;        o "AC Validate" reordered the buttons
; V1.8 - 27/NOV/2020
;        o "AC Validate": disables the gadgets before processing and enables after
;        o "AC Validate": Add text to buffer string to avoid refreshing the EditorGadget and cause a large CPU load
; V1.9 - 08/APR/2023
;        o Autocorrect Validate: removed the useless requirement of needing a dictionary to use it.
;
Procedure ac_validate()
      
    If number_of_autocorrect_words=0
      MessageRequester("Warning!","No autocorrect words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf  
    
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found."+#LF$+#LF$+"A dictionary is needed to check if the"+#LF$+"correct entries exist in the Autocorrect List."+#LF$+#LF$+"Dictionary checks will be skipped.",#PB_MessageRequester_Warning)
    EndIf    
    
    
    If OpenWindow(#WINDOW_AC_VALIDATE,0,0,get_extra_width(100)+(1024-800)+(800-640)+640-10,10+get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,"Autocorrect Validate",#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    WindowBounds(#WINDOW_AC_VALIDATE,get_extra_width(100)+(1024-800)+(800-640)+640-10,get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,#PB_Ignore,#PB_Ignore) ; 14/FEB/2018
    DisableWindow(#WINDOW_MAIN,#True)    
    
    ; EditorGadget
    EditorGadget(#EDITOR_AC_VALIDATE,10,10,WindowWidth(#WINDOW_AC_VALIDATE)-20,40-10+400+get_extra_height(100),#PB_Editor_ReadOnly)
    ac_validate_header$="Autocorrect Validate V2.1 - 2023-04-08"+#LF$
    ac_validate_header$+"============================"+#LF$
    ac_validate_header$+""+#LF$
    ac_validate_header$+"This tool will analyse the Autocorrect list finding issues, such as entries"+#LF$
    ac_validate_header$+"replacing with words that do not exist in the dictionary and entries that"+#LF$
    ac_validate_header$+"replace with incorrect entries: ('color', 'colour'), ('colour', 'color')."+#LF$
    ac_validate_header$+""+#LF$+#LF$
    SetGadgetText(#EDITOR_AC_VALIDATE,ac_validate_header$)
    SetGadgetFont(#EDITOR_AC_VALIDATE,FontID(#font_for_language_id))

    
    ; CLOSE  button
    x=WindowWidth(#WINDOW_AC_VALIDATE)-90-10
    y=WindowHeight(#WINDOW_AC_VALIDATE)-25-10-10
    ButtonGadget(#BUTTON_AC_VALIDATE_CLOSE,x,y,90,default_gadget_height_button,"Close")
    
    x=GadgetX(#EDITOR_AC_VALIDATE)
    y=GadgetY(#EDITOR_AC_VALIDATE)+GadgetHeight(#EDITOR_AC_VALIDATE)+10
    
    ; Process button
    ButtonGadget(#BUTTON_AC_VALIDATE_PROCESS,x,y,90,default_gadget_height_button,"Process")
            
    ; Export button
    x=GadgetX(#BUTTON_AC_VALIDATE_PROCESS)
    ButtonGadget(#BUTTON_AC_VALIDATE_EXPORT,x+GadgetWidth(#BUTTON_AC_VALIDATE_PROCESS)+10,y,90,default_gadget_height_button,"Export")
    
    ; Clear button
    ButtonImageGadget(#BUTTON_AC_VALIDATE_CLEAR,GadgetX(#BUTTON_AC_VALIDATE_EXPORT)+GadgetWidth(#BUTTON_AC_VALIDATE_EXPORT)+16-4-2,y,default_gadget_height_button,default_gadget_height_button,ImageID(101))
    GadgetToolTip(#BUTTON_AC_VALIDATE_CLEAR,"Clear")
    
    ; BOTTOM button
    ButtonImageGadget(#BUTTON_AC_VALIDATE_BOTTOM,GadgetX(#BUTTON_AC_VALIDATE_CLEAR)+GadgetWidth(#BUTTON_AC_VALIDATE_CLEAR)+16+16-8,y,default_gadget_height_button,default_gadget_height_button,ImageID(37))    
    GadgetToolTip(#BUTTON_AC_VALIDATE_BOTTOM,"Go to bottom")
    
    ; TOP button
    ButtonImageGadget(#BUTTON_AC_VALIDATE_TOP,GadgetX(#BUTTON_AC_VALIDATE_BOTTOM)+GadgetWidth(#BUTTON_AC_VALIDATE_BOTTOM)+16-4-2,y,default_gadget_height_button,default_gadget_height_button,ImageID(94))
    GadgetToolTip(#BUTTON_AC_VALIDATE_TOP,"Go to top")
    
    
    y+25+5+5+5-3
    
    x=GadgetX(#EDITOR_AC_VALIDATE)
    
    y+5
    
    ; Processing X of Y
    TextGadget(#TEXT_AC_VALIDATE_PROCESS,x,y,200,default_gadget_height_text,"Processing:")
    GadgetFit(#TEXT_AC_VALIDATE_PROCESS)
    ProgressBarGadget(#PROGRESSBAR_AC_VALIDATE,GadgetX(#TEXT_AC_VALIDATE_PROCESS)+GadgetWidth(#TEXT_AC_VALIDATE_PROCESS)+5,y,default_gadget_width_progress,default_gadget_height_progress,0,number_of_words)
    
    
    ; <ESC> to abort scan
    TextGadget(#TEXT_AC_VALIDATE_ESC_TO_ABORT,GadgetX(#PROGRESSBAR_AC_VALIDATE),GadgetY(#PROGRESSBAR_AC_VALIDATE)+GadgetHeight(#PROGRESSBAR_AC_VALIDATE)+1,100,default_gadget_height_text,"")       

    
    ; Status of Processing: "Done" or "Aborted"
    x=GadgetX(#PROGRESSBAR_AC_VALIDATE)+GadgetWidth(#PROGRESSBAR_AC_VALIDATE)+5
    TextGadget(#TEXT_AC_VALIDATE_PROCESS_DONE,x,y,100,16,"")
    SetGadgetColor(#TEXT_AC_VALIDATE_PROCESS_DONE,#PB_Gadget_FrontColor,#Black)
    
    
    x=GadgetX(#EDITOR_AC_VALIDATE)+GadgetWidth(#EDITOR_AC_VALIDATE)/2
    y=GadgetY(#EDITOR_AC_VALIDATE)+GadgetHeight(#EDITOR_AC_VALIDATE)+5+3
    
    
    ; Number of issues found
    TextGadget(#TEXT_AC_VALIDATE_ISSUES_FOUND,x,y,240,default_gadget_height_text,"Issues:0")
    
    y+20
       
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_AC_VALIDATE,#PB_Shortcut_Escape,1000)    
    
    close=#False
    Repeat
      
      event=WaitWindowEvent()
      
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        SetGadgetText(#TEXT_AC_VALIDATE_ESC_TO_ABORT,"")
        close=#True
      EndIf
      
      
      Select event 
        Case #PB_Event_Gadget
          Select EventGadget()
              
            Case #BUTTON_AC_VALIDATE_CLOSE
              close=#True
              
            Case #BUTTON_AC_VALIDATE_PROCESS
              SetGadgetText(#TEXT_AC_VALIDATE_PROCESS_DONE,"")
              SetGadgetColor(#TEXT_AC_VALIDATE_PROCESS_DONE,#PB_Gadget_FrontColor,#Black)
              SetGadgetText(#TEXT_AC_VALIDATE_ESC_TO_ABORT,"<ESC> to abort.")
              ac_validate_disable_gadgets_on_off(#True)
              ac_validate_process()        
              ac_validate_disable_gadgets_on_off(#False)
              SetGadgetText(#TEXT_AC_VALIDATE_ESC_TO_ABORT,"")
              
            Case #BUTTON_AC_VALIDATE_CLEAR
              SetGadgetText(#EDITOR_AC_VALIDATE,ac_validate_header$)
              
            Case #BUTTON_AC_VALIDATE_EXPORT
               flag=#False
               file$=SaveFileRequester("Export results to file","","Text Documents (*.txt)|*.txt",0)
               If file$<>""
                 If LCase(Right(file$,4))<>".txt" : file$+".txt": EndIf
                 
                 t=ReadFile(1,file$)
                 If t<>0
                   CloseFile(1)
                   If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
                       DeleteFile(file$) ; Delete .TXT file                 
                  Else
                       flag=#True ; Abort the save as the file already exists and pressed to not overwrite it
                  EndIf  
                EndIf    
                 
                If flag=#False
                    ; Saving the .TXT file
                    t$=ConvertStringToUnix(GetGadgetText(#EDITOR_AC_VALIDATE))
                    OpenFile(1,file$)
                      WriteStringFormat(1,#PB_UTF8)
                      WriteString(1,t$,#PB_UTF8)
                    CloseFile(1)
                  
                 EndIf
                   
               EndIf
               
            Case #BUTTON_AC_VALIDATE_BOTTOM
              SetPosBottom(#EDITOR_AC_VALIDATE,1)
               
            Case #BUTTON_AC_VALIDATE_TOP
              SetPosTop(#EDITOR_AC_VALIDATE,1)              
              
          EndSelect
          
      EndSelect
      
    Until close=#True
    
    
    CloseWindow(#WINDOW_AC_VALIDATE)
    DisableWindow(#WINDOW_MAIN,#False)  
  
EndProcedure




; AC Validate Process
;
; To search for autocorrects not existing in the dictionary, plus invalid autocorrects
;
; Decodes the word list and then analyse it
;
; V1.0 - 24/SEP/2018
;        25/SEP/2018
; V1.1 - 04/SEP/2018
;        o Major speed up in "AC Validate" in Ubuntu, thanks to the forum user #NULL.
; V1.2 - 27/NOV/2020
;        o "AC Validate": Add text to buffer string to avoid refreshing the EditorGadget and cause a large CPU load
; V1.3 - 29/NOV/2020
;        o "AC Validate": Ask if continue when extracting words goes beyond the safe value of the prefs
; V1.4 - 21/DEC/2020
;        o Prefs: "0" disables above N words warning
; V1.5 - 08/APR/2023
;        o Autocorrect Validate: removed the useless requirement of needing a dictionary to use it.
;
Procedure ac_validate_process()
  
    issues=0
  
    AddLineEditorGadget(#EDITOR_AC_VALIDATE,"Step 1/4 - Extracting dictionary wordlist...")
    ac_validate_process_current_text_buffer_in_steps$=""
    
    If number_of_words=0 : AddLineEditorGadget(#EDITOR_AC_VALIDATE,"No dictionary found - skipping dictionary checks..."+#LF$+#LF$+#LF$) : EndIf
    
If number_of_words<>0
    ; Step 1 (extracting wordlist)
    AddWindowTimer(#WINDOW_AC_VALIDATE,800,800)
    time=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
    total_number_of_processed_words=0
    Dim total_number_of_processed_words_array$(0)
    ask=#False
    
    SetGadgetAttribute(#PROGRESSBAR_AC_VALIDATE,#PB_ProgressBar_Maximum,number_of_words-1)
    SetGadgetState(#PROGRESSBAR_AC_VALIDATE,0)
    
    For f=0 To number_of_words-1
      
      text$=StringField(words$(f),1,#LF$)

      total_number_of_processed_words+process_pfx_sfx(4,text$,#False,#False,#False,"","","",#False,#False)

                      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800        
        SetGadgetState(#PROGRESSBAR_AC_VALIDATE,f)
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetState(#PROGRESSBAR_AC_VALIDATE,0)
          AddLineEditorGadget(#EDITOR_AC_VALIDATE,"Aborted.")
          SetGadgetText(#TEXT_AC_VALIDATE_PROCESS_DONE,"✗")
          SetGadgetColor(#TEXT_AC_VALIDATE_PROCESS_DONE,#PB_Gadget_FrontColor,#Red)
          RemoveWindowTimer(#WINDOW_AC_VALIDATE,800)
          ProcedureReturn #False
       EndIf
      EndIf
      
      
      ; Ask to abort if over N words
      If ask=#False And total_number_of_processed_words>=ask_after_n_words And ask_after_n_words<>0
        ask=#True
        If MessageRequester("Warning!","Over "+FormatNumber(ask_after_n_words,0,".",number_separator_chr$)+" words have been decoded."+#LF$+#LF$+"This represents a huge amount of data in the RAM"+#LF$+"that can fill it up and crash the computer."+#LF$+#LF$+"Abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          AddLineEditorGadget(#EDITOR_AC_VALIDATE,"Failed to extract full .dic wordlist.")
          ask=1000 ; Answered "yes" on abort
          Break
        EndIf
      EndIf         
      
      
    Next f
    
    
    If ask<>1000 : AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"Done!") : EndIf
    AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"Total wordlist:"+Str(total_number_of_processed_words))
    AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"")
    AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"")
    AddLineEditorGadget(#EDITOR_AC_VALIDATE,ac_validate_process_current_text_buffer_in_steps$)
EndIf    
    
    
    ; Step 2 (extracting autocorrect wordlist)
    
    AddLineEditorGadget(#EDITOR_AC_VALIDATE,"Step 2/4 - Extracting autocorrect list...")
    ac_validate_process_current_text_buffer_in_steps$=""
    
    time=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
    Dim total_number_of_processed_autocorrect_words_array$(number_of_autocorrect_words-1)
    Dim total_number_of_processed_autocorrect_words_array_incorrect$(number_of_autocorrect_words-1)
    
    SetGadgetAttribute(#PROGRESSBAR_AC_VALIDATE,#PB_ProgressBar_Maximum,number_of_autocorrect_words-1)
    SetGadgetState(#PROGRESSBAR_AC_VALIDATE,0)
    
    For f=0 To number_of_autocorrect_words-1
      
      ; Copy the Autocorrect list to the words processed array
      text$=StringField(autocorrect_words$(f),1,#LF$)
      text2$=StringField(autocorrect_words$(f),2,#LF$)
      If Left(text$,1)=":" And Right(text$,1)=":" : text2$="" : EndIf
      total_number_of_processed_autocorrect_words_array$(f)=text2$
      total_number_of_processed_autocorrect_words_array_incorrect$(f)=text$

      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        SetGadgetState(#PROGRESSBAR_AC_VALIDATE,f)
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
        If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          SetGadgetState(#PROGRESSBAR_AC_VALIDATE,0)
          AddLineEditorGadget(#EDITOR_AC_VALIDATE,"Aborted.")
          SetGadgetText(#TEXT_AC_VALIDATE_PROCESS_DONE,"✗")
          SetGadgetColor(#TEXT_AC_VALIDATE_PROCESS_DONE,#PB_Gadget_FrontColor,#Red)
          RemoveWindowTimer(#WINDOW_AC_VALIDATE,800)
          ProcedureReturn #False
       EndIf
      EndIf
        

    Next f    
    
    AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"Done!")
    AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"Total autocorrect list:"+Str(number_of_autocorrect_words))
    AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"")
    AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"")
    AddLineEditorGadget(#EDITOR_AC_VALIDATE,ac_validate_process_current_text_buffer_in_steps$)
    

    ; Step 3 (Comparing extracted autocorrect wordlist with dictionary)
    
    AddLineEditorGadget(#EDITOR_AC_VALIDATE,"Step 3/4 - Comparing extracted autocorrect wordlist with dictionary...")
    ac_validate_process_current_text_buffer_in_steps$=""    
    
    If number_of_words=0 : AddLineEditorGadget(#EDITOR_AC_VALIDATE,"No dictionary found - skipping dictionary checks..."+#LF$+#LF$+#LF$) : EndIf
    
If number_of_words<>0
    time=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
    
    SetGadgetAttribute(#PROGRESSBAR_AC_VALIDATE,#PB_ProgressBar_Maximum,number_of_autocorrect_words-1)
    SetGadgetState(#PROGRESSBAR_AC_VALIDATE,0)
    
    For f=0 To number_of_autocorrect_words-1
      
      If ask=1000
        AddLineEditorGadget(#EDITOR_AC_VALIDATE,"Dictionary words extraction aborted by user, so skipping this step.")
        Break
      EndIf
      
      found=#False
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        SetGadgetState(#PROGRESSBAR_AC_VALIDATE,f)
      EndIf       
      
      For ff=0 To total_number_of_processed_words-1
                
        ; Check if autocorrect is found in dictionary
        If total_number_of_processed_autocorrect_words_array$(f)=total_number_of_processed_words_array$(ff)
          found=#True
          Break
        EndIf
        
        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event = WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
          SetGadgetState(#PROGRESSBAR_AC_VALIDATE,f)
        EndIf             
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
          If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            SetGadgetState(#PROGRESSBAR_AC_VALIDATE,0)
            AddLineEditorGadget(#EDITOR_AC_VALIDATE,"Aborted.")
            SetGadgetText(#TEXT_AC_VALIDATE_PROCESS_DONE,"✗")
            SetGadgetColor(#TEXT_AC_VALIDATE_PROCESS_DONE,#PB_Gadget_FrontColor,#Red)
            RemoveWindowTimer(#WINDOW_AC_VALIDATE,800)
            ProcedureReturn #False
          EndIf
        EndIf
        
      Next ff
      
      ; Autocorrect entry not found
      If found=#False And total_number_of_processed_autocorrect_words_array$(f)<>""
        AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"Not found:"+"'"+total_number_of_processed_autocorrect_words_array$(f)+"'"+"→Line "+Str(f+1))
        issues+1
        SetGadgetText(#TEXT_AC_VALIDATE_ISSUES_FOUND,"Issues:"+Str(issues))
      EndIf
      
    Next f    
    
    SetGadgetState(#PROGRESSBAR_AC_VALIDATE,0)    
    If ask<>1000 : AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"Done!") : EndIf
    AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"")
    AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"")
    AddLineEditorGadget(#EDITOR_AC_VALIDATE,ac_validate_process_current_text_buffer_in_steps$)
EndIf

    ; Step 4 (Comparing extracted autocorrect wordlist entries: incorrect > correct)
    
    AddLineEditorGadget(#EDITOR_AC_VALIDATE,"Step 4/4 - Comparing extracted autocorrect wordlist with incorrect/correct...")
    ac_validate_process_current_text_buffer_in_steps$=""
    
    time=ElapsedMilliseconds()
    FUCKTIME=ElapsedMilliseconds()
    
    SetGadgetAttribute(#PROGRESSBAR_AC_VALIDATE,#PB_ProgressBar_Maximum,number_of_autocorrect_words-1)
    SetGadgetState(#PROGRESSBAR_AC_VALIDATE,0)
    
    For f=0 To number_of_autocorrect_words-1
      
      ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        SetGadgetState(#PROGRESSBAR_AC_VALIDATE,f)
      EndIf 
      
      
      For ff=0 To number_of_autocorrect_words-1
                
        ; Check if autocorrect is entry is wrong in incorrect/correct
        If total_number_of_processed_autocorrect_words_array$(f)=total_number_of_processed_autocorrect_words_array_incorrect$(ff)
          AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"Incorrect Autocorrect entry:"+"'"+total_number_of_processed_autocorrect_words_array$(f)+"','"+total_number_of_processed_autocorrect_words_array_incorrect$(f)+"'"+"→Line "+Str(f+1))
          issues+1
          SetGadgetText(#TEXT_AC_VALIDATE_ISSUES_FOUND,"Issues:"+Str(issues))
        EndIf
      
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
          If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            SetGadgetState(#PROGRESSBAR_AC_VALIDATE,0)
            AddLineEditorGadget(#EDITOR_AC_VALIDATE,"Aborted.")
            SetGadgetText(#TEXT_AC_VALIDATE_PROCESS_DONE,"✗")
            SetGadgetColor(#TEXT_AC_VALIDATE_PROCESS_DONE,#PB_Gadget_FrontColor,#Red)
            RemoveWindowTimer(#WINDOW_AC_VALIDATE,800)
            ProcedureReturn #False
          EndIf
        EndIf
        
      Next ff
      
      
    Next f    
    
    SetGadgetState(#PROGRESSBAR_AC_VALIDATE,0)    
    AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"Done!")
    AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"")
    AddLineEditorGadgetBuffer_AC(#EDITOR_AC_VALIDATE,"")
    AddLineEditorGadget(#EDITOR_AC_VALIDATE,ac_validate_process_current_text_buffer_in_steps$)
    SetGadgetText(#TEXT_AC_VALIDATE_PROCESS_DONE,"✓")
    SetGadgetColor(#TEXT_AC_VALIDATE_PROCESS_DONE,#PB_Gadget_FrontColor,#GreenDark)
    
    
    RemoveWindowTimer(#WINDOW_MAIN,800)
    
EndProcedure




  ; Disables the gadgets before processing and enables after
  ;
  ; V1.0 - 27/NOV/2020
  ;
  Procedure ac_validate_disable_gadgets_on_off(is_disable)

    DisableGadget(#BUTTON_AC_VALIDATE_CLOSE,is_disable)
    DisableGadget(#BUTTON_AC_VALIDATE_PROCESS,is_disable)
    DisableGadget(#BUTTON_AC_VALIDATE_EXPORT,is_disable) 
    DisableGadget(#BUTTON_AC_VALIDATE_CLEAR,is_disable)
    DisableGadget(#BUTTON_AC_VALIDATE_BOTTOM,is_disable)
    DisableGadget(#BUTTON_AC_VALIDATE_TOP,is_disable)

  EndProcedure
  
  
  
  
  ; Add text to buffer string to avoid refreshing the EditorGadget and cause a large CPU load
  ;
  ; V1.0 - 27/NOV/2020
  ;
  Procedure AddLineEditorGadgetBuffer_AC(gadget,text$)
    ac_validate_process_current_text_buffer_in_steps$+text$+#LF$
  EndProcedure
  
; IDE Options = PureBasic 6.04 LTS (Windows - x64)
; CursorPosition = 360
; FirstLine = 345
; Folding = -----
; EnableXP
; CompileSourceDirectory
; EnableUnicode