﻿  ; Load LanguageTool grammar.xml
  ;
  ; V1.0 - 25/DEC/2016
  ;        o Just placed a warning window saying it is not coded yet.
  ; V1.1 - 01/DEC/2017
  ;        o Message Requesters now have a sign.
  ; V1.2 - 15/DEC/2020
  ;        24/DEC/2020
  ;        27/DEC/2020
  ;        28/DEC/2020
  ;        29/DEC/2020
  ;        30/DEC/2020
  ;        06/JAN/2021
  ;        12/JAN/2021
  ;        16/JAN/2021
  ;        o Basic support to open a grammar.xml file
  ; V1.3 - 08/APR/2021
  ;        o LanguageTool rules now display contents
  ; V1.4 - 12/APR/2021
  ;        o Improved LanguageTool rules decoding.  
  ; V1.5 - 06/MAY/2021
  ;        o Optimised loading speed of LanguageTool rules.
  ; V1.6 - 02/JUL/2021
  ;        o Now the sizes of load/save main files for each tab shows the files label. Ex: "(aff:1 MB)+(dic: 10MB)"
  ; V1.7 - 22/NOV/2021
  ;        23/NOV/2021
  ;        o Started showing the time taken in timely operations.  
  ; V1.8 - 13/FEB/2023
  ;        o Fixed the "LanguageTool" processing that would not recognise some rules after a category.
  ; V1.9 - 29/JUN/2023
  ;        o LanguageTool: opening a file now shows % progress.
  ; V2.0 - 08/DEC/2023
  ;        o LanguageTool: Now shows the number of rules per category
  ;
  Procedure load_languagetool(force_file$)
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Loading grammar rules... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
    
    
    If force_file$=""
      file$=OpenFileRequester("Open grammar rules", "", "Grammar.xml (*.xml)|*.xml", 0)
      If file$=""
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")  
        ProcedureReturn #False
      EndIf
    Else
      file$=force_file$
    EndIf      
    
    
    ; Check if .DIC file exists
    If IsFileExist(file$,"",".XML file not found.")=#False
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "File not found.")
      ProcedureReturn #False
    EndIf    
    
    time_taken = ElapsedMilliseconds()  
    
    
    ; Clear the contents of the editor gadget, rule ID and rule name
    SetGadgetText(#EDITOR_GADGET_PANEL_BOX_LANGUAGETOOL_RULES_BODY,"")
    SetGadgetText(#TEXT_PANEL_BOX_LANGUAGETOOL_RULE_ID,"Rule ID:n/a")
    GadgetFit(#TEXT_PANEL_BOX_LANGUAGETOOL_RULE_ID)
    SetGadgetText(#TEXT_PANEL_BOX_LANGUAGETOOL_RULE_TEXT,"Rule Name:n/a")
    GadgetFit(#TEXT_PANEL_BOX_LANGUAGETOOL_RULE_TEXT)
    

    ; LOAD RULES
    Dim lines_lt_buffer$(0)
    number_of_lines_lt_buffer=0
    ClearGadgetItems(#PANEL_BOX_LANGUAGETOOL)
    ClearGadgetItems(#PANEL_BOX_LANGUAGETOOL_RIGHT)
    languagetool_total_categories_update()
    HideGadget(#TEXT_PANEL_BOX_LANGUAGETOOL_NUMBER_WORDS_ASTERISK,#True)
    ; Read .XML file
    AddWindowTimer(#WINDOW_MAIN,800,800)      
    ReadFile(1,File$)  
      ReadStringFormat(1)
      text$=ReadString(1,#PB_UTF8) ;Just to read the header of the .XML which contains the rules
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,Lof(1))
      Repeat
        word$=ReadString(1,#PB_UTF8)
        number_of_lines_lt_buffer+1
        ReDim lines_lt_buffer$(number_of_lines_lt_buffer-1)
        lines_lt_buffer$(number_of_lines_lt_buffer-1)=word$

        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event=WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Loading grammar rules... <ESC> to abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken)+ConvertProgressToPercentage(Loc(1),Lof(1),1))
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,Loc(1),#PB_StatusBar_Raised)
        EndIf 
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
         If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            CloseFile(1)
            RemoveWindowTimer(#WINDOW_MAIN,800)
            number_of_laguagetools=0
            ProcedureReturn #False
         EndIf
        EndIf

      Until Eof(1)
    filesize_lof=Lof(1)
    CloseFile(1)  
    
    
    ; No lines found in file loaded above
    If number_of_lines_lt_buffer=0
      MessageRequester("Warning!","No rules found.",#PB_MessageRequester_Warning)
      number_of_laguagetools=0
      ProcedureReturn #False
    EndIf
    
    
    ; PROCESS RULES
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Processing grammar rules... <ESC> to abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
    number_of_lines_lt=0
    languagetools_categories_counter=0
    ; Process .XML file
    AddWindowTimer(#WINDOW_MAIN,800,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised,0,number_of_lines_lt_buffer)
    counter=0
    rule_text_contents$=""
    counter_rule_rulegroup_by_category=0 ; To get the number of rules per category
    counter_rule_rulegroup_by_category_all=0 ; Sum of all rules of the categories
    Repeat
        word$=lines_lt_buffer$(number_of_lines_lt)
        number_of_lines_lt+1
        ReDim languagetools(counter)
        
        t$=""
;         t=FindString(LCase(word$),LCase("<rulegroup id="))
        t=FindString(LCase(word$),LCase("<rulegroup "))
        If t>0          
          t$=Mid(word$,t)
          languagetools(counter)\chunk_type.s="rulegroup"
;           languagetools(counter)\id.s=ExtractStringFromQuotes(word$,t+Len("<rulegroup id="))
          t=FindString(LCase(word$),LCase("id="))
          languagetools(counter)\id.s=ExtractStringFromQuotes(word$,t+Len("id="))
          t=FindString(LCase(word$),LCase("name="))
          languagetools(counter)\name.s=ExtractStringFromQuotes(word$,t+Len("name="))          
          t=FindString(LCase(word$),LCase("type="))
          languagetools(counter)\type.s=ExtractStringFromQuotes(word$,t+Len("type="))          
          t=FindString(LCase(word$),LCase("tags="))
          languagetools(counter)\tags.s=ExtractStringFromQuotes(word$,t+Len("tags="))          
          t=FindString(LCase(word$),LCase("default="))
          languagetools(counter)\default_.s=ExtractStringFromQuotes(word$,t+Len("default="))
          If counter>0
            languagetools(counter-1)\rule_text.s=rule_text_contents$
            rule_text_contents$=""
          EndIf   
          counter_rule_rulegroup_by_category+1
        EndIf
        
        t=FindString(LCase(word$),LCase("<rule "))
        If t>0
          t=FindString(LCase(word$),LCase("id="))
          t$=Mid(word$,t)
          languagetools(counter)\chunk_type.s="rule"
          languagetools(counter)\id.s=ExtractStringFromQuotes(word$,t+Len("id="))
          t=FindString(LCase(word$),LCase("name="))
          languagetools(counter)\name.s=ExtractStringFromQuotes(word$,t+Len("name="))
          t=FindString(LCase(word$),LCase("type="))
          languagetools(counter)\type.s=ExtractStringFromQuotes(word$,t+Len("type="))          
          t=FindString(LCase(word$),LCase("tags="))
          languagetools(counter)\tags.s=ExtractStringFromQuotes(word$,t+Len("tags="))                    
          t=FindString(LCase(word$),LCase("default="))
          languagetools(counter)\default_.s=ExtractStringFromQuotes(word$,t+Len("default="))                 
          If counter>0
            languagetools(counter-1)\rule_text.s=rule_text_contents$
            rule_text_contents$=""
          EndIf    
          counter_rule_rulegroup_by_category+1
        EndIf
        
        t=FindString(LCase(word$),LCase("<category id="))
        If t>0
          t$=Mid(word$,t)
          languagetools(counter)\chunk_type.s="category"
          languagetools(counter)\id.s=ExtractStringFromQuotes(word$,t+Len("<category id="))
          t=FindString(LCase(word$),LCase("name="))
          languagetools(counter)\name.s=ExtractStringFromQuotes(word$,t+Len("name="))
          t=FindString(LCase(word$),LCase("type="))
          languagetools(counter)\type.s=ExtractStringFromQuotes(word$,t+Len("type="))          
          t=FindString(LCase(word$),LCase("tags="))
          languagetools(counter)\tags.s=ExtractStringFromQuotes(word$,t+Len("tags="))          
          t=FindString(LCase(word$),LCase("default="))
          languagetools(counter)\default_.s=ExtractStringFromQuotes(word$,t+Len("default="))                 
          If counter>0
            languagetools(counter-1)\rule_text.s=rule_text_contents$
            rule_text_contents$=""
          EndIf          
        EndIf
        
        If t$<>""
          languagetools(counter)\line_number.l=number_of_lines_lt
          ttt$=Str(languagetools(counter)\line_number.l)+#LF$+languagetools(counter)\chunk_type.s+#LF$+languagetools(counter)\id.s+#LF$+languagetools(counter)\name.s+#LF$+languagetools(counter)\type.s+#LF$+languagetools(counter)\tags.s+#LF$+languagetools(counter)\default_.s
          counter+1
          AddGadgetItem(#PANEL_BOX_LANGUAGETOOL,-1,ttt$)
          SetGadgetItemColor(#PANEL_BOX_LANGUAGETOOL,counter-1,#PB_Gadget_BackColor,#GreyLight,0)            
          colour=#Black
          If languagetools(counter-1)\chunk_type.s="rulegroup" : colour=#GreenDark : EndIf
          If languagetools(counter-1)\chunk_type.s="category" : colour=#Brown : EndIf
          If colour<>#Black
            SetGadgetItemColor(#PANEL_BOX_LANGUAGETOOL,counter-1,#PB_Gadget_FrontColor,colour,1)
            SetGadgetItemColor(#PANEL_BOX_LANGUAGETOOL,counter-1,#PB_Gadget_FrontColor,colour,2)
            SetGadgetItemColor(#PANEL_BOX_LANGUAGETOOL,counter-1,#PB_Gadget_FrontColor,colour,3)
            SetGadgetItemColor(#PANEL_BOX_LANGUAGETOOL,counter-1,#PB_Gadget_FrontColor,colour,4)
            SetGadgetItemColor(#PANEL_BOX_LANGUAGETOOL,counter-1,#PB_Gadget_FrontColor,colour,5)
            SetGadgetItemColor(#PANEL_BOX_LANGUAGETOOL,counter-1,#PB_Gadget_FrontColor,colour,6)            
          EndIf
          If languagetools(counter-1)\default_.s="off" : SetGadgetItemColor(#PANEL_BOX_LANGUAGETOOL,counter-1,#PB_Gadget_FrontColor,#Red,6) : EndIf
          If colour=#Brown
            ttt$=Str(languagetools(counter-1)\line_number.l)+#LF$+languagetools(counter-1)\id.s
            AddGadgetItem(#PANEL_BOX_LANGUAGETOOL_RIGHT,-1,ttt$)
            SetGadgetItemColor(#PANEL_BOX_LANGUAGETOOL_RIGHT,#PB_All,#PB_Gadget_BackColor,#GreyLight,0)
            SetGadgetItemColor(#PANEL_BOX_LANGUAGETOOL_RIGHT,#PB_All,#PB_Gadget_FrontColor,colour,1)
            ReDim languagetools_categories(languagetools_categories_counter)
            languagetools_categories(languagetools_categories_counter)=counter-1
            languagetools_categories_counter+1  
            
            ; Add the number of rules per category on right panel
            t=CountGadgetItems(#PANEL_BOX_LANGUAGETOOL_RIGHT)
            If t>1
              t-1-1
              counter_rule_rulegroup_by_category_all+counter_rule_rulegroup_by_category
              SetGadgetItemText(#PANEL_BOX_LANGUAGETOOL_RIGHT,t,GetGadgetItemText(#PANEL_BOX_LANGUAGETOOL_RIGHT,t,1)+" ("+Str(counter_rule_rulegroup_by_category)+")",1)
              counter_rule_rulegroup_by_category=0
            EndIf
          EndIf
         
        EndIf
        
        ; Add contents to rule
        rule_text_contents$+word$+#LF$
        
        ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
        event=WindowEvent()
        If event=#PB_Event_Timer And EventTimer()=800
          SetGadgetText(#TEXT_PANEL_BOX_LANGUAGETOOL_NUMBER_WORDS,"# Chunks:"+Str(counter))
          GadgetFit(#TEXT_PANEL_BOX_LANGUAGETOOL_NUMBER_WORDS)
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Processing grammar rules... <ESC> to abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken)+ConvertProgressToPercentage(number_of_lines_lt,number_of_lines_lt_buffer,1))
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,number_of_lines_lt,#PB_StatusBar_Raised)
        EndIf 
        
        ; <ESC> pressed to abort
        If EventMenu()=1000
         If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
            CloseFile(1)
            RemoveWindowTimer(#WINDOW_MAIN,800)
            number_of_laguagetools=counter
            ProcedureReturn #False
         EndIf
        EndIf

    Until number_of_lines_lt>=number_of_lines_lt_buffer ; It is better to use >= just in case we don't know if it matches

    ; Add the number of rules per category on right panel to last category
    t=CountGadgetItems(#PANEL_BOX_LANGUAGETOOL_RIGHT)
    If t>1
      t-1
      SetGadgetItemText(#PANEL_BOX_LANGUAGETOOL_RIGHT,t,GetGadgetItemText(#PANEL_BOX_LANGUAGETOOL_RIGHT,t,1)+" ("+Str(counter_rule_rulegroup_by_category)+")",1)
      counter_rule_rulegroup_by_category_all+counter_rule_rulegroup_by_category
      counter_rule_rulegroup_by_category=0
    EndIf

    
    
    ; Add contents to last rule
    If counter>0
      languagetools(counter-1)\rule_text.s=rule_text_contents$
      rule_text_contents$=""
    EndIf    
    
    RemoveWindowTimer(#WINDOW_MAIN,800)
    SetGadgetText(#TEXT_PANEL_BOX_LANGUAGETOOL_NUMBER_WORDS,"# Chunks:"+Str(counter))
    number_of_laguagetools=counter
    GadgetFit(#TEXT_PANEL_BOX_LANGUAGETOOL_NUMBER_WORDS)    
    
    languagetool_filename$=file$
    update_filename_languagetool()
    
    ; 31/OCT/2017 - Support for "Recent Files" menu entry
    create_load_update_recent_files(file$)
    update_recent_files_menu()
    
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK. "+ShowFileSizeFormatted("xml:",filesize_lof,#True)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken)) 
    
  EndProcedure 
  
  
  
  
  ; Updates the filename of grammar.xml in LanguageTool
  ;
  ; Used in open and save as
  ;
  ; V1.0 - 12/JAN/2021
  ; V1.1 - 19/DEC/2021
  ;        o Replaced "Filename:" with "File:"
  ; V1.2 - 01/NOV/2023
  ;        o Replaced "File:" with "💾:"
  ; V1.3 - 26/NOV/2023
  ;        o Replaced the disk emoji plain text with an image
  ; V1.4 - 13/JAN/2024
  ;        14/JAN/2024
  ;        o It now shows a folder emoji with the path of the filename
  ;
  Procedure update_filename_languagetool()
    
    TextReplaceIfDifferentGadget(#TEXT_PANEL_BOX_LANGUAGETOOL_FILENAME,""+GetOnlyFilename(languagetool_filename$,"n/a"),#True)
    TextReplaceIfDifferentGadget(#TEXT_PANEL_BOX_LANGUAGETOOL_FOLDER,""+GetOnlyPath(languagetool_filename$,"n/a"),#True)
    
  EndProcedure  
  
  
  
  
  ; Extract POS settings
  ;
  ; V1.0 - 19/MAY/2020
  ;        20/MAY/2020
  ;        23/MAY/2020
  ;        25/MAY/2020
  ;        27/MAY/2020
  ;        28/MAY/2020  
  ;        29/MAY/2020  
  ;        30/MAY/2020
  ;        31/MAY/2020
  ;        06/JUN/2020
  ;        19/OCT/2020
  ;        20/OCT/2020
  ;        03/NOV/2020
  ;
  Procedure languagetool_POS_settings()
    
    If OpenWindow(#WINDOW_LANGUAGETOOL_POS_SETTINGS,0,0,get_extra_width(100)+(1024-800)+(800-640)+640-10,get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,"Extract POS Settings (Unfinished)",#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
    WindowBounds(#WINDOW_LANGUAGETOOL_POS_SETTINGS,get_extra_width(100)+(1024-800)+(800-640)+640-10,get_extra_height(100)+200-50-20-20+100+100+200-30-30-30+10+120,#PB_Ignore,#PB_Ignore) ; 14/FEB/2018
    DisableWindow(#WINDOW_MAIN,#True)    
    
    x=10
    y=10
    
    ; Use language
    TextGadget(#TEXT_LANGUAGETOOL_POS_SETTINGS_USE_LANGUAGE,x,y,100,default_gadget_height_text,"Use language:")
    GadgetFit(#TEXT_LANGUAGETOOL_POS_SETTINGS_USE_LANGUAGE)
    y+default_gadget_height_text+5    
    ComboBoxGadget(#COMBOBOX_LANGUAGETOOL_POS_SETTINGS_USE_LANGUAGE,x,y,100+15+15,default_gadget_height_combobox)
    AddGadgetItem(#COMBOBOX_LANGUAGETOOL_POS_SETTINGS_USE_LANGUAGE,-1,languagetool_pos_rules_languages$(0))
    AddGadgetItem(#COMBOBOX_LANGUAGETOOL_POS_SETTINGS_USE_LANGUAGE,-1,languagetool_pos_rules_languages$(1))
    AddGadgetItem(#COMBOBOX_LANGUAGETOOL_POS_SETTINGS_USE_LANGUAGE,-1,languagetool_pos_rules_languages$(2))
    SetGadgetState(#COMBOBOX_LANGUAGETOOL_POS_SETTINGS_USE_LANGUAGE,languagetool_pos_rules_language_selected)
    
    
    ; Remember button
    ButtonGadget(#BUTTON_LANGUAGETOOL_POS_SETTINGS_REMEMBER,GadgetX(#COMBOBOX_LANGUAGETOOL_POS_SETTINGS_USE_LANGUAGE)+GadgetWidth(#COMBOBOX_LANGUAGETOOL_POS_SETTINGS_USE_LANGUAGE)+15,GadgetY(#COMBOBOX_LANGUAGETOOL_POS_SETTINGS_USE_LANGUAGE),80+8,default_gadget_height_button,"Remember")   
    
    
    y+default_gadget_height_combobox+15
    
    
    ; ListIcon gadget    
    ListIconGadget(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,x,y,7+3+3+20+60-150+1200+20+30+get_extra_width(100)-200,25+25+500-200+get_extra_height(100),"POS source: .dic",15+10+50+200+get_extra_width(10),#PB_ListIcon_FullRowSelect|#PB_ListIcon_AlwaysShowSelection|#PB_ListIcon_CheckBoxes)
    AddGadgetColumn(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,1,"POS target: LanguageTool",15+10+50+200+get_extra_width(10))
    AddGadgetColumn(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,2,"Comments",100+50+300+get_extra_width(10))
    SetGadgetFont(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,FontID(#font_for_language_id))    
    
    ; Add the buttons under the GadgetList
    y=GadgetY(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)+GadgetHeight(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)+10
    y-2
    ButtonGadget(#BUTTON_LANGUAGETOOL_POS_SETTINGS_ADD,10,y,90,default_gadget_height_button,"Add")
    ButtonGadget(#BUTTON_LANGUAGETOOL_POS_SETTINGS_EDIT,10+100-5,y,90,default_gadget_height_button,"Edit")
    ButtonGadget(#BUTTON_LANGUAGETOOL_POS_SETTINGS_DELETE,10+100+100-5-5,y,90,default_gadget_height_button,"Delete")    
    
    ; BOTTOM button EditorGadget gadget
    ButtonImageGadget(#BUTTON_LANGUAGETOOL_POS_SETTINGS_BOTTOM,GadgetX(#BUTTON_LANGUAGETOOL_POS_SETTINGS_DELETE)+GadgetWidth(#BUTTON_LANGUAGETOOL_POS_SETTINGS_DELETE)+16+16-8,GadgetY(#BUTTON_LANGUAGETOOL_POS_SETTINGS_DELETE),default_gadget_height_button,default_gadget_height_button,ImageID(37))            
    GadgetToolTip(#BUTTON_LANGUAGETOOL_POS_SETTINGS_BOTTOM,"Go to bottom")       
    
    ; TOP button EditorGadget gadget
    ButtonImageGadget(#BUTTON_LANGUAGETOOL_POS_SETTINGS_TOP,GadgetX(#BUTTON_LANGUAGETOOL_POS_SETTINGS_BOTTOM)+GadgetWidth(#BUTTON_LANGUAGETOOL_POS_SETTINGS_BOTTOM)+16-4-2,GadgetY(#BUTTON_LANGUAGETOOL_POS_SETTINGS_BOTTOM),default_gadget_height_button,default_gadget_height_button,ImageID(94))
    GadgetToolTip(#BUTTON_LANGUAGETOOL_POS_SETTINGS_TOP,"Go to top")
    
    ; OK button
    ButtonGadget(#BUTTON_LANGUAGETOOL_POS_SETTINGS_OKAY,WindowWidth(#WINDOW_LANGUAGETOOL_POS_SETTINGS)-100-15,-15+WindowHeight(#WINDOW_LANGUAGETOOL_POS_SETTINGS)-20,90,default_gadget_height_button,"OK")       
    
    ; Cancel button
    ButtonGadget(#BUTTON_LANGUAGETOOL_POS_SETTINGS_CANCEL,5+GadgetX(#BUTTON_LANGUAGETOOL_POS_SETTINGS_OKAY)-100-15,-15+WindowHeight(#WINDOW_LANGUAGETOOL_POS_SETTINGS)-20,90,default_gadget_height_button,"Cancel")         
    
    
    x=GadgetX(#BUTTON_LANGUAGETOOL_POS_SETTINGS_CANCEL)-GadgetWidth(#BUTTON_LANGUAGETOOL_POS_SETTINGS_CANCEL)-15-5
    y=GadgetY(#BUTTON_LANGUAGETOOL_POS_SETTINGS_CANCEL)
    
    
    ; Save button
    ButtonGadget(#BUTTON_LANGUAGETOOL_POS_SETTINGS_SAVE,x,y,90,default_gadget_height_button,"Save")    
    
    
    x=GadgetX(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)
    y=GadgetY(#BUTTON_LANGUAGETOOL_POS_SETTINGS_ADD)+GadgetHeight(#BUTTON_LANGUAGETOOL_POS_SETTINGS_ADD)+3+3+3
    
    
    ; Number of lines added
    TextGadget(#BUTTON_LANGUAGETOOL_POS_SETTINGS_INDEX_TOTAL,x,y,200,default_gadget_height_text,"Found:0")
    GadgetFit(#BUTTON_LANGUAGETOOL_POS_SETTINGS_INDEX_TOTAL)
    
    
    y+default_gadget_height_text+3+3
    
    
    ; Selected
    TextGadget(#BUTTON_LANGUAGETOOL_POS_SETTINGS_INDEX_SELECTED,x,y,200,default_gadget_height_text,"Selected:None")
    GadgetFit(#BUTTON_LANGUAGETOOL_POS_SETTINGS_INDEX_SELECTED)
    
    
    ; Create Popup menus
    CreatePopupImageMenu(#POPUPMENU_POS_SETTINGS,#PB_Menu_ModernLook)
    MenuItem(#POPUPMENU_POS_SETTINGS_ADD_ABOVE,"Add Above",ImageID(90))    
    MenuItem(#POPUPMENU_POS_SETTINGS_ADD_BELOW,"Add Below",ImageID(91))    
    MenuItem(#POPUPMENU_POS_SETTINGS_EDIT,"Edit",ImageID(38))
    MenuItem(#POPUPMENU_POS_SETTINGS_CLONE_BELOW,"Clone Below",ImageID(92))    
    MenuBar()
    MenuItem(#POPUPMENU_POS_SETTINGS_SELECT_ALL,"Select All",ImageID(42))
    MenuItem(#POPUPMENU_POS_SETTINGS_SELECT_NONE,"Select None",ImageID(43))
    MenuItem(#POPUPMENU_POS_SETTINGS_SELECT_INVERT,"Invert Selection",ImageID(47))
    MenuBar()
    MenuItem(#POPUPMENU_POS_SETTINGS_DELETE_SELECTED,"Delete Selected",ImageID(44))
    MenuItem(#POPUPMENU_POS_SETTINGS_DELETE,"Delete",ImageID(39))
    
    
    ; Populate the ListIconGadget
    languagetool_POS_settings_populate(languagetool_pos_rules_language_selected)
    
    
    ; Update total items
    languagetool_POS_settings_update_total_selected(languagetool_pos_rules_language_selected)
    
    
    ; Add keyboard shortcuts
    AddKeyboardShortcut(#WINDOW_LANGUAGETOOL_POS_SETTINGS,#PB_Shortcut_Escape,1000)
    AddKeyboardShortcut(#WINDOW_LANGUAGETOOL_POS_SETTINGS,#PB_Shortcut_Delete,1001)
    AddKeyboardShortcut(#WINDOW_LANGUAGETOOL_POS_SETTINGS,#PB_Shortcut_F1,1004)   
    AddKeyboardShortcut(#WINDOW_LANGUAGETOOL_POS_SETTINGS,#PB_Shortcut_Control|#PB_Shortcut_A,1014)
    
    
    ; Get old settings in case CANCEL is pressed
    old_languagetool_pos_rules_language_selected=languagetool_pos_rules_language_selected
    Dim old_languagetool_pos_rules_language_selected_rules_number_max_each(languagetool_pos_rules_language_selected_total_number_languages-1)
    CopyArray(languagetool_pos_rules_language_selected_rules_number_max_each(),old_languagetool_pos_rules_language_selected_rules_number_max_each())
    Dim old_languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected_total_number_languages-1,#languagetool_pos_rules_language_selected_rules_number_max)
    CopyArray(languagetool_pos_rules_language_selected_rules$(),old_languagetool_pos_rules_language_selected_rules$())    
    
    
    okay=#False
    cancel=#False
    
    Repeat
      
      event=WaitWindowEvent()
      
      
      ; Pressed the close window gadget (Ubuntu)
      If event=#PB_Event_CloseWindow
        cancel=#True     
      EndIf     
      
      
      ; Check if <ESC> was pressed
      If event=#PB_Event_Menu And EventMenu()=1000
        Debug "<ESC> was pressed"
        cancel=#True
      EndIf     
      
      
      ; Check if DEL was pressed
      If event=#PB_Event_Menu And EventMenu()=1001
        Debug "DEL was pressed"
        languagetool_POS_settings_delete()
        languagetool_POS_settings_update_total_selected(languagetool_pos_rules_language_selected)
      EndIf           
      
      
      ; Check if CTR+A was pressed
      If event=#PB_Event_Menu And EventMenu()=1014
        Debug "CTR+A was pressed"
        PostEvent(#PB_Event_Gadget, #WINDOW_LANGUAGETOOL_POS_SETTINGS,#BUTTON_LANGUAGETOOL_POS_SETTINGS_ADD)
      EndIf              
      
      
      ; Shortcut for F1 (help)
      If event=#PB_Event_Menu And EventMenu()=1004
        html_help("")
      EndIf   
      
      
      ; Double-click on ListIconGadget
      If EventGadget()=#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS And EventType()=#PB_EventType_LeftDoubleClick
        t=GetGadgetState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)
        If t=-1
          MessageRequester("Warning!","No rule selected.",#PB_MessageRequester_Warning)
        Else
          t$=languagetool_POS_settings_edit(GetGadgetItemText(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,t,0)+#LF$+GetGadgetItemText(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,t,1)+#LF$+GetGadgetItemText(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,t,2))
          If t$<>""
            languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,t+1)=t$
            SetGadgetItemText(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,t,t$)
          EndIf
        EndIf 
      EndIf
      
      
      ; Clicked on a checkbox of a line
      If EventType()=#PB_EventType_Change And EventGadget()=#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS
        Debug "check/uncheck item"
        languagetool_POS_settings_update_total_selected(languagetool_pos_rules_language_selected)
      EndIf
        
      
      ; POP-UP MENU
      If EventGadget()=#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS And EventType()=#PB_EventType_RightClick
        DisplayPopupMenu(#POPUPMENU_POS_SETTINGS, WindowID(#WINDOW_LANGUAGETOOL_POS_SETTINGS))
      EndIf  
      
      Select event
  
      Case #PB_Event_Menu
      
        Select EventMenu()  ; To see which popup menu has been selected
            
          ; ADD ABOVE
          Case #POPUPMENU_POS_SETTINGS_ADD_ABOVE
              t$=languagetool_POS_settings_edit(#LF$+#LF$) 

              If t$<>""
                line_selected=GetGadgetState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)
                For f=languagetool_pos_rules_language_selected_rules_number_max_each(languagetool_pos_rules_language_selected) To line_selected Step -1
                  languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,f+1)=languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,f)
                Next f
                languagetool_pos_rules_language_selected_rules_number_max_each(languagetool_pos_rules_language_selected)+1
                languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,line_selected+1)=t$
                AddGadgetItem(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,line_selected,t$)
                languagetool_POS_settings_update_total_selected(languagetool_pos_rules_language_selected)
              EndIf                    
            
          ; ADD BELOW
          Case #POPUPMENU_POS_SETTINGS_ADD_BELOW
              t$=languagetool_POS_settings_edit(#LF$+#LF$) 

              If t$<>""
                line_selected=GetGadgetState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)+1
                For f=languagetool_pos_rules_language_selected_rules_number_max_each(languagetool_pos_rules_language_selected) To line_selected Step -1
                  languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,f+1)=languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,f)
                Next f
                languagetool_pos_rules_language_selected_rules_number_max_each(languagetool_pos_rules_language_selected)+1
                languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,line_selected+1)=t$
                AddGadgetItem(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,line_selected,t$)
                languagetool_POS_settings_update_total_selected(languagetool_pos_rules_language_selected)
              EndIf                
            
          Case #POPUPMENU_POS_SETTINGS_EDIT
            PostEvent(#PB_Event_Gadget, #WINDOW_GOTO, #BUTTON_LANGUAGETOOL_POS_SETTINGS_EDIT)
              
          Case #POPUPMENU_POS_SETTINGS_CLONE_BELOW  
              counter=CountGadgetItems(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)
            
              t=GetGadgetState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)
              If t=-1
                MessageRequester("Warning!","No rule selected.",#PB_MessageRequester_Warning)
              Else
                t$=GetGadgetItemText(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,t,0)+#LF$+GetGadgetItemText(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,t,1)+#LF$+GetGadgetItemText(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,t,2)
                line_selected=GetGadgetState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)+1
                For f=languagetool_pos_rules_language_selected_rules_number_max_each(languagetool_pos_rules_language_selected) To line_selected Step -1
                  languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,f+1)=languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,f)
                Next f
                languagetool_pos_rules_language_selected_rules_number_max_each(languagetool_pos_rules_language_selected)+1
                languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,line_selected+1)=t$
                AddGadgetItem(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,line_selected,t$)
                languagetool_POS_settings_update_total_selected(languagetool_pos_rules_language_selected)                
              EndIf 
            
          Case #POPUPMENU_POS_SETTINGS_SELECT_ALL
            counter=CountGadgetItems(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)
            If counter>0
              For f=0 To counter-1
                SetGadgetItemState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,f,#PB_ListIcon_Checked)
              Next f
            EndIf   
            languagetool_POS_settings_update_total_selected(languagetool_pos_rules_language_selected)
              
          Case #POPUPMENU_POS_SETTINGS_SELECT_NONE
            counter=CountGadgetItems(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)
            If counter>0
              For f=0 To counter-1
                SetGadgetItemState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,f,#False)
              Next f
            EndIf      
            languagetool_POS_settings_update_total_selected(languagetool_pos_rules_language_selected)
            
          Case #POPUPMENU_POS_SETTINGS_SELECT_INVERT
            counter=CountGadgetItems(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)
            If counter=0 : MessageRequester("Warning!","No items selected.",#PB_MessageRequester_Warning) : EndIf
            If counter>0
              For f=0 To counter-1
                If GetGadgetItemState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,f) & #PB_ListIcon_Checked
                  SetGadgetItemState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,f,#False)
                Else
                  SetGadgetItemState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,f,#PB_ListIcon_Checked)
                EndIf
              Next f
            EndIf
            languagetool_POS_settings_update_total_selected(languagetool_pos_rules_language_selected)
            
          Case #POPUPMENU_POS_SETTINGS_DELETE_SELECTED
            counter=CountGadgetItems(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)
            If counter=0 : MessageRequester("Warning!","No items selected.",#PB_MessageRequester_Warning) : EndIf
            If counter>0
              
              Dim buffer$(#languagetool_pos_rules_language_selected_rules_number_max)
              flag=#False 
              For f=counter-1 To 0 Step -1
                buffer$(f+1)=languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,f+1)
                If GetGadgetItemState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,f) & #PB_ListIcon_Checked
                  flag=#True
                  RemoveGadgetItem(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,f)
                  buffer$(f+1)=""
                EndIf                
              Next f
              If flag=#False : MessageRequester("Warning!","No items selected.",#PB_MessageRequester_Warning) : EndIf
              If flag=#True
                current_rule=1
                For f=1 To counter
                  If buffer$(f)<>""
                    languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,current_rule)=buffer$(f)
                    current_rule+1
                  EndIf
                Next f                             
                
                ; Remove rules from the rules list
                languagetool_pos_rules_language_selected_rules_number_max_each(languagetool_pos_rules_language_selected)=CountGadgetItems(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)
                
              EndIf
              
            EndIf
            languagetool_POS_settings_update_total_selected(languagetool_pos_rules_language_selected)         
            
            
          Case #POPUPMENU_POS_SETTINGS_DELETE
            PostEvent(#PB_Event_Gadget, #WINDOW_GOTO, #BUTTON_LANGUAGETOOL_POS_SETTINGS_DELETE)
            
        EndSelect
        
      EndSelect
    
      
      Select event 
        Case #PB_Event_Gadget
          Select EventGadget()
              
           ; Change current use language
           Case #COMBOBOX_LANGUAGETOOL_POS_SETTINGS_USE_LANGUAGE
             If EventType()=#PB_EventType_Change And languagetool_pos_rules_language_selected<>GetGadgetState(#COMBOBOX_LANGUAGETOOL_POS_SETTINGS_USE_LANGUAGE)
                languagetool_pos_rules_language_selected=GetGadgetState(#COMBOBOX_LANGUAGETOOL_POS_SETTINGS_USE_LANGUAGE)
                languagetool_POS_settings_populate(languagetool_pos_rules_language_selected)    
                languagetool_POS_settings_update_total_selected(languagetool_pos_rules_language_selected)
             EndIf
             
             
           ; Press REMEMBER
           Case #BUTTON_LANGUAGETOOL_POS_SETTINGS_REMEMBER           
            ; Delete old .prefs file    
            If FileSize(main_path$+"settings/"+"extract_pos_settings.prefs")>=0
              DeleteFile(main_path$+"settings/"+"extract_pos_settings.prefs")
            EndIf               
            
            ; Attempt to save languagetool_pos_rules_language_selected .PREFS file
            OpenFile(1,main_path$+"settings/"+"extract_pos_settings.prefs")  
              WriteString(1,prefs_file_version$+#LF$,#PB_UTF8)
              WriteLong(1,languagetool_pos_rules_language_selected)                                
            CloseFile(1)
            
            
            Case #BUTTON_LANGUAGETOOL_POS_SETTINGS_BOTTOM
              SetGadgetState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,CountGadgetItems(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)-1)  
              
              
            Case #BUTTON_LANGUAGETOOL_POS_SETTINGS_TOP
              SetGadgetState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,0)   
              
              
           ; DELETE
           Case #BUTTON_LANGUAGETOOL_POS_SETTINGS_DELETE
              languagetool_POS_settings_delete()
              languagetool_POS_settings_update_total_selected(languagetool_pos_rules_language_selected)
              
              
           ; ADD
           Case #BUTTON_LANGUAGETOOL_POS_SETTINGS_ADD
              t$=languagetool_POS_settings_edit(#LF$+#LF$)
              If t$<>""
                languagetool_pos_rules_language_selected_rules_number_max_each(languagetool_pos_rules_language_selected)+1
                languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,languagetool_pos_rules_language_selected_rules_number_max_each(languagetool_pos_rules_language_selected))=t$
                AddGadgetItem(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,-1,t$)
                languagetool_POS_settings_update_total_selected(languagetool_pos_rules_language_selected)
              EndIf
              
              
           ; EDIT
           Case #BUTTON_LANGUAGETOOL_POS_SETTINGS_EDIT
              t=GetGadgetState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)
              If t=-1
                MessageRequester("Warning!","No rule selected.",#PB_MessageRequester_Warning)
              Else
                t$=languagetool_POS_settings_edit(GetGadgetItemText(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,t,0)+#LF$+GetGadgetItemText(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,t,1)+#LF$+GetGadgetItemText(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,t,2))
                If t$<>""
                  languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,t+1)=t$
                  SetGadgetItemText(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,t,t$)
                EndIf
              EndIf 

              
           ; SAVE lists
           Case #BUTTON_LANGUAGETOOL_POS_SETTINGS_SAVE              
            ; Delete old .prefs file    
            If FileSize(main_path$+"settings/"+"extract_pos_lists.prefs")>=0
              DeleteFile(main_path$+"settings/"+"extract_pos_lists.prefs")
            EndIf               
            
            ; Attempt to save languagetool_pos_rules_language_selected .PREFS file
            OpenFile(1,main_path$+"settings/"+"extract_pos_lists.prefs")    
              WriteString(1,prefs_file_version$+#LF$,#PB_UTF8)
              WriteString(1,Str(languagetool_pos_rules_language_selected_total_number_languages)+#LF$,#PB_UTF8) ; Total number of languages in combobox
              For f=1 To languagetool_pos_rules_language_selected_total_number_languages
                WriteString(1,languagetool_pos_rules_languages$(f-1)+#LF$,#PB_UTF8) ; Languages names in combobox
                counter=languagetool_pos_rules_language_selected_rules_number_max_each(f-1)
                WriteString(1,Str(counter)+#LF$,#PB_UTF8) ; Total number of rules of language F
                If counter>0
                  For ff=1 To counter
                    WriteString(1,ReplaceString(languagetool_pos_rules_language_selected_rules$(f-1,ff),#LF$,#TAB$)+#LF$,#PB_UTF8) ; Each rule of language FF
                  Next ff
                EndIf
              Next f
            CloseFile(1)
             
             
           ; OK
          Case #BUTTON_LANGUAGETOOL_POS_SETTINGS_OKAY
            If old_languagetool_pos_rules_language_selected<>languagetool_pos_rules_language_selected
              SetMenuItemText(#MENU_MAIN,#MENU_LANGUAGETOOL_EXTRACT_POS_ADDEDTXT,"POS added.txt "+"("+languagetool_pos_rules_languages$(languagetool_pos_rules_language_selected)+")")
              SetMenuItemText(#MENU_MAIN,#MENU_LANGUAGETOOL_EXTRACT_POS_SPELLINGTXT,"POS spelling.txt "+"("+languagetool_pos_rules_languages$(languagetool_pos_rules_language_selected)+")")
            EndIf
             okay=#True
             
             
           ; CANCEL  
           Case #BUTTON_LANGUAGETOOL_POS_SETTINGS_CANCEL
             languagetool_pos_rules_language_selected=old_languagetool_pos_rules_language_selected
             CopyArray(old_languagetool_pos_rules_language_selected_rules_number_max_each(),languagetool_pos_rules_language_selected_rules_number_max_each())
             CopyArray(old_languagetool_pos_rules_language_selected_rules$(),languagetool_pos_rules_language_selected_rules$())
             cancel=#True
             
         EndSelect
         
       EndSelect                 
       
       
    Until okay=#True Or cancel=#True

    
    CloseWindow(#WINDOW_LANGUAGETOOL_POS_SETTINGS)
    DisableWindow(#WINDOW_MAIN,#False)  
        
  EndProcedure
  
  
  
  
  ; Populates the ListIconGadget based on the language in the combobox
  ;
  ; V1.0 - 19/MAY/2020
  ;  
  Procedure languagetool_POS_settings_populate(language_selected)
    
    ; Clear the ListIconGadget
    ClearGadgetItems(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)
    
    
    ; Populate
    counter=languagetool_pos_rules_language_selected_rules_number_max_each(language_selected)
    
    ; No entries
    If counter=0 : ProcedureReturn : EndIf
    
    ; Entries
    For f=1 To counter
      AddGadgetItem(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,-1,languagetool_pos_rules_language_selected_rules$(language_selected,f))
    Next f
    
  EndProcedure
  
  
  
  
  ; Updates the total of items plus the number of selected items
  ;
  ; V1.0 - 31/MAY/2020
  ;        20/OCT/2020
  ;        03/NOV/2020
  ;  
  Procedure languagetool_POS_settings_update_total_selected(language_selected)

    ; Total items
    counter=languagetool_pos_rules_language_selected_rules_number_max_each(language_selected)
    If GetGadgetText(#BUTTON_LANGUAGETOOL_POS_SETTINGS_INDEX_TOTAL)<>"Found:"+Str(counter)
      SetGadgetText(#BUTTON_LANGUAGETOOL_POS_SETTINGS_INDEX_TOTAL,"Found:"+Str(counter))
      GadgetFit(#BUTTON_LANGUAGETOOL_POS_SETTINGS_INDEX_TOTAL)
    EndIf
    
    ; Total Selected
    counter=CountGadgetItems(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)
    If counter=0 : t$="None" : EndIf
    If counter>0
      t=0      
      For f=0 To counter-1
        If GetGadgetItemState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,f) & #PB_ListIcon_Checked : t+1 : EndIf
      Next f
      t$=Str(t)+"/"+Str(languagetool_pos_rules_language_selected_rules_number_max_each(language_selected))
      If t=0 : t$="None" : EndIf
      If t=counter : t$="All" : EndIf
    EndIf
    If GetGadgetText(#BUTTON_LANGUAGETOOL_POS_SETTINGS_INDEX_SELECTED)<>"Selected:"+t$
      SetGadgetText(#BUTTON_LANGUAGETOOL_POS_SETTINGS_INDEX_SELECTED,"Selected:"+t$)
      GadgetFit(#BUTTON_LANGUAGETOOL_POS_SETTINGS_INDEX_SELECTED)
    EndIf
 
  EndProcedure  
  
  
  
  
  ; Defaults the rules.
  ;
  ; Used before attempted to load saved rules.
  ;
  ; V1.0 - 19/MAY/2020
  ;        20/MAY/2020
  ;        29/MAY/2020
  ;        30/MAY/2020
  ;        06/JUN/2020
  ;        09/JUN/2020
  ;        13/JUL/2020
  ;        15/JAN/2021
  ;        25/JAN/2021
  ;        27/JAN/2021
  ;        02/FEB/2021
  ;        07/FEB/2021
  ;
  Procedure languagetool_POS_settings_rules_default()
    
    
  ; Languages in Combo Box
  languagetool_pos_rules_languages$(0)="Other"
  languagetool_pos_rules_languages$(1)="Portuguese"
  languagetool_pos_rules_languages$(2)="English"
    
    
  ; Other (blank)
  counter=0
  
  ; Number of rules of this language
  languagetool_pos_rules_language_selected_rules_number_max_each(0)=counter  
  
  
  ; Portuguese
  counter=0
  
  ; STARTING FEMALE PT  
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[CAT=np,G=f,SEM=p]"+#LF$+"NPFS000"+#LF$+"Proper name - Female"
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[CAT=np,G=f]"+#LF$+"NPFS000"+#LF$+"Proper name - Female"  
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[CAT=np,N=s,G=f]"+#LF$+"NPFS000"+#LF$+"Proper name - Female"  
      
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[LA=1,CAT=np,SEM=country,G=f,N=s]"+#LF$+"NPFSG00"+#LF$+"Place name - Country - Female" 
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[CAT=np,SEM=country,G=f,N=s]"+#LF$+"NPFSG00"+#LF$+"Place name - Country - Female" 
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[LA=1,CAT=np,SEM=country,G=f,N=s,N=p]"+#LF$+"NPFPG00"+#LF$+"Place name - Country - Female" 
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[CAT=np,SEM=country,G=f,N=s,N=p]"+#LF$+"NPFPG00"+#LF$+"Place name - Country - Female" 
    
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[LA=1,CAT=np,SEM=ter,G=f,N=s]"+#LF$+"NPFSG00"+#LF$+"Place name - City - Female"
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[CAT=np,SEM=ter,G=f,N=s]"+#LF$+"NPFSG00"+#LF$+"Place name - City - Female"   
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[CAT=np,SEM=cid,G=f,N=s]"+#LF$+"NPFSG00"+#LF$+"Place name - City - Female"   
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[LA=1,CAT=np,SEM=cid,G=f,N=s]"+#LF$+"NPFSG00"+#LF$+"Place name - City - Female"     
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[LA=1,CAT=np,SEM=ter,G=f,N=p]"+#LF$+"NPFPG00"+#LF$+"Place name - Place - Female"    
  
  ; STARTING MALE PT

  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[CAT=np,G=m,SEM=p]"+#LF$+"NPMS000"+#LF$+"Proper name - Male"
    
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[CAT=np,G=m]"+#LF$+"NPMS000"+#LF$+"Proper name - Male"
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[CAT=np,G=m,SEM=p1]"+#LF$+"NPMS000"+#LF$+"Proper name - Male"  
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[LA=1,CAT=np,SEM=country,G=m,N=s]"+#LF$+"NPMSG00"+#LF$+"Place name - Country - Male" 
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[CAT=np,SEM=country,G=m,N=s]"+#LF$+"NPMSG00"+#LF$+"Place name - Country - Male"   
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[LA=1,CAT=np,SEM=ter,G=m,N=s]"+#LF$+"NPMSG00"+#LF$+"Place name - City - Male"
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[CAT=np,SEM=ter,G=m,N=s]"+#LF$+"NPMSG00"+#LF$+"Place name - City - Male"  
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[CAT=np,SEM=cid,G=m,N=s]"+#LF$+"NPMSG00"+#LF$+"Place name - City - Male"    
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[LA=1,CAT=np,SEM=cid,G=m,N=s]"+#LF$+"NPMSG00"+#LF$+"Place name - City - Male"    
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[LA=1,CAT=np,SEM=ter,G=m,N=p]"+#LF$+"NPMPG00"+#LF$+"Place name - Place - Male"    
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(1,counter)="[LA=1,CAT=np,SEM=rio]"+#LF$+"NPMS000"+#LF$+"Place name - River - Male"    
  
  ; Number of rules of this language
  languagetool_pos_rules_language_selected_rules_number_max_each(1)=counter
  
  
  ; English
  counter=0
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Adjective"+#LF$+"JJ"+#LF$+"Adjective: beautiful, large, inspectable"
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Adjective: comparative"+#LF$+"JJR"+#LF$+"Adjective, comparative: larger, quicker"  
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Adjective: superlative"+#LF$+"JJS"+#LF$+"Adjective, superlative: largest, quickest"
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Adverb"+#LF$+"RB"+#LF$+"Adverb and negation: easily, sunnily, suddenly, specifically, not"  
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Adverb: comparative"+#LF$+"RBR"+#LF$+"Adverb, comparative: better, faster, quicker"  
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Adverb: superlative"+#LF$+"RBS"+#LF$+"Adverb, superlative: best, fastest, quickest"  
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Noun: uncountable"+#LF$+"NN:U"+#LF$+"Noun: uncountable"
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic clade"+#LF$+"NNP"+#LF$+"Taxonomic clade"  
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic class"+#LF$+"NNP"+#LF$+"Taxonomic class"
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic division"+#LF$+"NNP"+#LF$+"Taxonomic division"
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic family"+#LF$+"NNP"+#LF$+"Taxonomic family"  
    
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic genus"+#LF$+"NNP"+#LF$+"Taxonomic genus"  
    
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic infraclass"+#LF$+"NNP"+#LF$+"Taxonomic infraclass"  
    
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic kingdom"+#LF$+"NNP"+#LF$+"Taxonomic kingdom"  
    
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic order"+#LF$+"NNP"+#LF$+"Taxonomic order"  
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic phylum"+#LF$+"NNP"+#LF$+"Taxonomic phylum"  
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic phylum + Taxonomic class"+#LF$+"NNP"+#LF$+"Taxonomic phylum + Taxonomic class"    
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic realm"+#LF$+"NNP"+#LF$+"Taxonomic realm"  
    
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic subclass"+#LF$+"NNP"+#LF$+"Taxonomic subclass"  
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic subfamily"+#LF$+"NNP"+#LF$+"Taxonomic subfamily"  

  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic subgenus"+#LF$+"NNP"+#LF$+"Taxonomic subgenus"  
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic subkingdom"+#LF$+"NNP"+#LF$+"Taxonomic subkingdom"  
    
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic suborder"+#LF$+"NNP"+#LF$+"Taxonomic suborder"  
    
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic subphylum"+#LF$+"NNP"+#LF$+"Taxonomic subphylum"  
    
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic superclass"+#LF$+"NNP"+#LF$+"Taxonomic superclass"  
    
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic superorder"+#LF$+"NNP"+#LF$+"Taxonomic superorder"  
  
  counter+1
  languagetool_pos_rules_language_selected_rules$(2,counter)="Taxonomic tribe"+#LF$+"NNP"+#LF$+"Taxonomic tribe"  
  
  
;   counter+1
;   languagetool_pos_rules_language_selected_rules$(2,counter)="Noun: usually uncountable"+#LF$+"NN:UN"+#LF$+"Noun: usually uncountable"  
  
  ; Number of rules of this language
  languagetool_pos_rules_language_selected_rules_number_max_each(2)=counter
  
  EndProcedure
  
  


; POS Extract Edit/Add
;
; Based on the AFF Validate code
;
; V1.0 - 27/MAY/2020
;        03/NOV/2020
;
Procedure.s languagetool_POS_settings_edit(fields$)
  
  ; If field$="" it is an Add, and should check if maximum reached
  If fields$="" And languagetool_pos_rules_language_selected_rules_number_max_each(languagetool_pos_rules_language_selected)+1>#languagetool_pos_rules_language_selected_rules_number_max
    MessageRequester("Warning!","The maximum number of rules per language is "+Str(#languagetool_pos_rules_language_selected_rules_number_max),#PB_MessageRequester_Warning)
    ProcedureReturn ""
  EndIf
  
  
  If OpenWindow(#WINDOW_LANGUAGETOOL_POS_EDIT,0,0,(800-640)+640-10-100,2+3-10-20-40-50-200+200-50-20-20+100+100+200-30-30-30+10+30+10,"Edit/Add a POS Setting line",#PB_Window_WindowCentered,WindowID(#WINDOW_MAIN))=0 : MessageRequester("Error", "Can't open a window.",#PB_MessageRequester_Error) : EndIf
  DisableWindow(#WINDOW_LANGUAGETOOL_POS_SETTINGS,#True)
        
  ; Source field
;   ImageGadget(#IMAGE_LANGUAGETOOL_POS_EDIT_SOURCE,0+6+3+2,1+1+1+2+0+6+2+1,16,16,ImageID(84))
  t$=StringField(fields$,1,#LF$)
  StringGadget(#STRING_LANGUAGETOOL_POS_EDIT_SOURCE,-25+10+0+6+100-15-15-20,6+0+6,20+320-10-20-40+20+30+30+60+70+70+70,default_gadget_height_string,t$)
  SetGadgetFont(#STRING_LANGUAGETOOL_POS_EDIT_SOURCE,FontID(#font_for_language_id))
  SetActiveGadget(#STRING_LANGUAGETOOL_POS_EDIT_SOURCE)
    
  ; Target field
;   ImageGadget(#IMAGE_LANGUAGETOOL_POS_EDIT_TARGET,0+6+3+2,1+1+1+2+6+0+6+2+1+20+5+2,16,16,ImageID(84))
  t$=StringField(fields$,2,#LF$)
  StringGadget(#STRING_LANGUAGETOOL_POS_EDIT_TARGET,-25+10+0+6+100-15-15-20,2+3+6+0+6+2+1+20+5+3,20+320-10-20-40+20+30+30+60+70+70+70,default_gadget_height_string,t$)      
  SetGadgetFont(#STRING_LANGUAGETOOL_POS_EDIT_TARGET,FontID(#font_for_language_id))

  ; Comments field
;   ImageGadget(#IMAGE_LANGUAGETOOL_POS_EDIT_COMMENTS,0+6+3+2,1+1+1+2+6+0+6+2+1+20+5+2+20+5+3+3+3+1+1,16,16,ImageID(84))
  t$=StringField(fields$,3,#LF$)
  StringGadget(#STRING_LANGUAGETOOL_POS_EDIT_COMMENTS,-25+10+0+6+100-15-15-20,2+3+6+0+6+2+1+20+5+20+5+20+5+3-7-5-2,20+320-10-20-40+20+30+30+60+70+70+70,default_gadget_height_string,t$)      
  SetGadgetFont(#STRING_LANGUAGETOOL_POS_EDIT_COMMENTS,FontID(#font_for_language_id))
  
  
  ; Add the OK and CANCEL buttons
  ButtonGadget(#BUTTON_LANGUAGETOOL_POS_EDIT_OKAY,0+6+100-15,2+3+10+30+30+3+2+20+5+7+7,90,default_gadget_height_button,"OK")
  ButtonGadget(#BUTTON_LANGUAGETOOL_POS_EDIT_CANCEL,0+6+100-15+100,2+3+10+30+30+3+2+20+5+7+7,90,default_gadget_height_button,"Cancel")
            
  ; Add keyboard shortcuts
  AddKeyboardShortcut(#WINDOW_LANGUAGETOOL_POS_EDIT,#PB_Shortcut_Escape,1000)    
  AddKeyboardShortcut(#WINDOW_LANGUAGETOOL_POS_EDIT,#PB_Shortcut_Return,1003)    
  AddKeyboardShortcut(#WINDOW_LANGUAGETOOL_POS_EDIT,#PB_Shortcut_F1,1004)
    
  okay=#False
  cancel=#False
  Repeat
        
    event=WaitWindowEvent()
      
      
    ; Pressed the close window gadget (Ubuntu)
    If event=#PB_Event_CloseWindow
      Debug "Close window pressed (Ubuntu)"
      cancel=#True     
    EndIf     
      
      
    ; Check if <ESC> was pressed
    If event=#PB_Event_Menu And EventMenu()=1000
      Debug "<ESC> was pressed"
      cancel=#True
    EndIf        
      
      
    ; Check if <ENTER> was pressed
    If event=#PB_Event_Menu And EventMenu()=1003
      Debug "OK was pressed"
      PostEvent(#PB_Event_Gadget, #WINDOW_LANGUAGETOOL_POS_EDIT, #BUTTON_LANGUAGETOOL_POS_EDIT_OKAY)
    EndIf      

   
    ; Shortcut for F1 (help)
    If event=#PB_Event_Menu And EventMenu()=1004
      html_help("")
    EndIf         
    
    
    Select Event 
      Case #PB_Event_Gadget
        Select EventGadget()
                                         
         ; Press OK
         Case #BUTTON_LANGUAGETOOL_POS_EDIT_OKAY
           okay=#True
           If Trim(GetGadgetText(#STRING_LANGUAGETOOL_POS_EDIT_SOURCE))="" Or Trim(GetGadgetText(#STRING_LANGUAGETOOL_POS_EDIT_TARGET))="" Or Trim(GetGadgetText(#STRING_LANGUAGETOOL_POS_EDIT_COMMENTS))=""
             MessageRequester("Warning!","Empty fields found.",#PB_MessageRequester_Warning)
             okay=#False
           EndIf
           
         ; Press CANCEL  
         Case #BUTTON_LANGUAGETOOL_POS_EDIT_CANCEL
           cancel=#True             
             
        EndSelect
     EndSelect                 
                 
      
  Until okay=#True Or cancel=#True
  
  t$=""
  ; OK button pressed. It will now add or update an autocorrect word    
  If okay=#True      
    t$=GetGadgetText(#STRING_LANGUAGETOOL_POS_EDIT_SOURCE)+#LF$+GetGadgetText(#STRING_LANGUAGETOOL_POS_EDIT_TARGET)+#LF$+GetGadgetText(#STRING_LANGUAGETOOL_POS_EDIT_COMMENTS)
  EndIf
    
    
  ; Close the Add/Edit window and activate the main window
  DisableWindow(#WINDOW_LANGUAGETOOL_POS_SETTINGS,#False)
  CloseWindow(#WINDOW_LANGUAGETOOL_POS_EDIT) 
  
  ProcedureReturn t$
  
EndProcedure 




; POS Extract Delete
;
;
; V1.0 - 28/MAY/2020
;
Procedure languagetool_POS_settings_delete()
  
  counter=CountGadgetItems(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)
  
  ; No words in current listicongadget
  If counter=0
    MessageRequester("Warning!","No rules for current language.",#PB_MessageRequester_Warning)
    ProcedureReturn        
  EndIf
  
  
  ; No item selected
  If GetGadgetState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)=-1
    MessageRequester("Warning!","No rule selected.",#PB_MessageRequester_Warning)
    ProcedureReturn        
  EndIf
  
  
  ; Confirm rule deletion
  If MessageRequester("Warning!","Really delete rule?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
    ProcedureReturn
  EndIf
  
  t=GetGadgetState(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS)
  Dim buffer$(#languagetool_pos_rules_language_selected_rules_number_max)
  For f=1 To counter
    buffer$(f)=languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,f)
  Next f
  buffer$(t+1)=""
  current_rule=1
  For f=1 To counter
    If buffer$(f)<>""
      languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,current_rule)=buffer$(f)
      current_rule+1
    EndIf
  Next f
  
  ; Remove one rule from the rules list
  languagetool_pos_rules_language_selected_rules_number_max_each(languagetool_pos_rules_language_selected)-1
  
  ; Remove rule from listicongadget
  RemoveGadgetItem(#LISTICON_LANGUAGETOOL_POS_SETTINGS_POS,t)
  
EndProcedure 


  

; Extract POS words
;
; added_or_spelling: 1=extract as added.txt  2=extract as spelling.txt
;
; V1.0 - 22/MAY/2020
;        25/MAY/2020
;        28/MAY/2020
; V1.1 - 02/DEC/2021
;        o Started showing the time taken in timely operations.
;
Procedure languagetool_POS_extract(added_or_spelling)
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Extracting POSes from Dictionary... <ESC> to abort.")
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) 
    
    If number_of_words=0
      MessageRequester("Warning!","No dictionary words found.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf
    
    t$=GetGadgetText(#EDITOR_GADGET_PANEL_BOX_WORDS_AFF)
    If t$=""
      MessageRequester("Warning!","The .AFF file is empty.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf    
    
    
    If languagetool_pos_rules_language_selected_rules_number_max_each(languagetool_pos_rules_language_selected)=0
      MessageRequester("Warning!","No POS rules for selected language.",#PB_MessageRequester_Warning)
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf    
    
    
    file$=SaveFileRequester("Save extracted POSes", "", "Text files (*.txt)|*.txt", 0)
    If file$=""
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
      ProcedureReturn #False
    EndIf
    If LCase(Right(file$,4))<>".txt" : file$+".txt": EndIf
    

    If FileSize(file$)>0
      If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.") 
        ProcedureReturn #False
      EndIf      
    EndIf    
    
    DeleteFile(file$) ; Delete .DIC file
    DeleteFile(Left(file$,Len(file$)-4)+".txt") ; Delete the .AFF file
    
    
    extracted_counter=0
    
    ; Saving the .txt with POSes file
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Extracting POSes from Dictionary... <ESC> to abort.")
    AddWindowTimer(#WINDOW_MAIN,800,800)
    OpenFile(1,File$)
      WriteStringFormat(1,#PB_UTF8)
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_words)     
      time_taken=ElapsedMilliseconds()
      For f=0 To number_of_words-1
        text$=words$(f)
        
        main_word$=StringField(text$,1,#LF$)
        main_word$=StringField(main_word$+"/",1,"/")
        main_pos$=StringField(text$,2,#LF$)
        
        
        For ff=1 To languagetool_pos_rules_language_selected_rules_number_max_each(languagetool_pos_rules_language_selected)
          
          source_pos$=Trim(StringField(languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,ff),1,#LF$))
          target_pos$=Trim(StringField(languagetool_pos_rules_language_selected_rules$(languagetool_pos_rules_language_selected,ff),2,#LF$))
          If main_pos$=source_pos$
            If added_or_spelling=1
              WriteString(1,main_word$+#TAB$+main_word$+#TAB$+target_pos$+#LF$,#PB_UTF8)
            Else
              WriteString(1,main_word$+#LF$,#PB_UTF8)
            EndIf
            extracted_counter+1
          EndIf
        
        
                      
          ; Fix to window stop responding if we click on it while adding synonyms, plus update the progress bar
          event = WindowEvent()
          If event=#PB_Event_Timer And EventTimer()=800
            StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Extracting POSes from Dictionary... <ESC> to abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
            StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
          EndIf 
        
          ; <ESC> pressed to abort
          If EventMenu()=1000
            If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
              StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
              StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
              CloseFile(1)
              RemoveWindowTimer(#WINDOW_MAIN,800)
              ProcedureReturn #False
            EndIf
          EndIf
          
        Next ff
        

      Next f
      RemoveWindowTimer(#WINDOW_MAIN,800)      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      CloseFile(1)
      
      
      StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) ; Reset progress bar - 27/MAR/2014
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK. POSes extracted:"+Str(extracted_counter)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))

EndProcedure




; Saves a .txt file with LanguageTool structure
;
; V1.0 - 21/JAN/2021
; V1.1 - 24/FEB/2021
;        o Trying to export empty rules structure now shows "Aborted."
; V1.2 - 02/DEC/2021
;        o Improved the LanguageTool Structure Extract
;        o Started showing the time taken in timely operations.
; V1.3 - 30/DEC/2021
;        o Improved the extract rules structure information
; V1.4 - 19/SEP/2022
;        o LanguageTool Structure Extract now supports extracting without line numbers
;
Procedure languagetool_structure_extract(extract_line_numbers)
  
  StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Exporting LanguageTool structure... <ESC> to abort.")
  StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised) 
  
  ; Check if no LanguageTool entries
  If number_of_laguagetools=0
    MessageRequester("Warning!","Nothing to save.",#PB_MessageRequester_Warning)  
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
    ProcedureReturn #False
  EndIf      
  
  
  ; Get Save .TXT filename
  file$=SaveFileRequester("Save Text File", "", "Text File (*.txt)|*.txt", 0)
  If file$=""
    ProcedureReturn #False
  EndIf
  If LCase(Right(file$,4))<>".txt" : file$+".txt": EndIf
  
  
  ; Check if file exists   
  If FileSize(file$)>0
    If MessageRequester("Warning!","File already exists! Overwrite it?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_No
      StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
      ProcedureReturn #False
    EndIf      
  EndIf      
  DeleteFile(file$) ; Delete .TXT file
  
  
  ; Save the .TXT file, using UTF-8 encoding
  StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Exporting LanguageTool structure... <ESC> to abort.")
  AddWindowTimer(#WINDOW_MAIN,800,800)  
  time_taken=ElapsedMilliseconds()  
  OpenFile(1,file$)
    WriteStringFormat(1,#PB_UTF8)
    
    ; Save the categories from the panel on the right
    If CountGadgetItems(#PANEL_BOX_LANGUAGETOOL_RIGHT)>0
      t$=""
      For f=0 To CountGadgetItems(#PANEL_BOX_LANGUAGETOOL_RIGHT)-1
        If extract_line_numbers=#True : t$+"Line: "+GetGadgetItemText(#PANEL_BOX_LANGUAGETOOL_RIGHT,f,0)+#LF$ : EndIf
        t$+"ID Category:"+GetGadgetItemText(#PANEL_BOX_LANGUAGETOOL_RIGHT,f,1)+#LF$+#LF$
      Next f
      WriteString(1,t$+#LF$+#LF$+#LF$,#PB_UTF8)
    EndIf
    
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised,0,number_of_laguagetools)     
    For f=1 To number_of_laguagetools
      
      
      ttt$=""
      If extract_line_numbers=#True : ttt$="Line: "+Str(languagetools(f-1)\line_number.l)+#LF$ : EndIf
      ttt$+languagetools(f-1)\chunk_type.s+", "+"ID:"+languagetools(f-1)\id.s+", "+"Name:"+Chr(34)+languagetools(f-1)\name.s+Chr(34)+", "+languagetools(f-1)\type.s+", "+languagetools(f-1)\tags.s+", "
      tttt$=languagetools(f-1)\default_.s      
      If tttt$<>"" : tttt$="default:"+tttt$ : EndIf
      ttt$+tttt$
        
      If f>1 And languagetools(f-1)\chunk_type.s="category" : ttt$=#LF$+#LF$+#LF$+ttt$ : EndIf ; Add another blank line before starting a category
      WriteString(1,ttt$+#LF$+#LF$,#PB_UTF8)
      
       ; Fix to window stop responding if we click on it while extracting, plus update the progress bar
      event = WindowEvent()
      If event=#PB_Event_Timer And EventTimer()=800
        StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Exporting LanguageTool structure... <ESC> to abort."+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
        StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,f,#PB_StatusBar_Raised)
      EndIf 
        
      ; <ESC> pressed to abort
      If EventMenu()=1000
       If MessageRequester("Warning!","Really abort?",#PB_MessageRequester_YesNo|#PB_MessageRequester_Warning)=#PB_MessageRequester_Yes
          StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "Aborted.")
          StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
          CloseFile(1)
          RemoveWindowTimer(#WINDOW_MAIN,800)           
          ProcedureReturn #False
       EndIf
      EndIf
      
    Next f
    
    RemoveWindowTimer(#WINDOW_MAIN,800)      
    StatusBarProgress(#WINDOW_MAIN_STATUS_BAR,1,0,#PB_StatusBar_Raised)
    filesize_structure_lt=Lof(1)
    CloseFile(1)  
    
    StatusBarText(#WINDOW_MAIN_STATUS_BAR, 0, "OK. "+ShowFileSizeFormatted("",filesize_structure_lt,#True)+TimeToString(" ⌚",ElapsedMilliseconds()-time_taken))
  
  EndProcedure
  
  
  
  
  ; LanguageTool tab: the ListIconGadget on the right shows total of categories found if >0
  ;
  ; V1.0 - 15/FEB/2023
  ; V1.1 - 27/DEC/2023
  ;        o Now shows total of rules for all categories
  ;
  Procedure languagetool_total_categories_update()
    
    counter=CountGadgetItems(#PANEL_BOX_LANGUAGETOOL_RIGHT)
  
    t$="ID Category "
    If counter>0 : t$+"("+Str(counter)+")"+", ∑ Rules ("+Str(counter_rule_rulegroup_by_category_all)+")" : EndIf
  
    SetGadgetItemText(#PANEL_BOX_LANGUAGETOOL_RIGHT,-1,t$,1)
  
  EndProcedure
  
  

; IDE Options = PureBasic 6.04 LTS (Windows - x64)
; CursorPosition = 319
; FirstLine = 301
; Folding = --
; EnableXP
; CompileSourceDirectory
; EnableUnicode